// Copyright (c) 2018 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// GENERATED FROM THE API DEFINITION IN
//   chrome/common/extensions/api/system_private.json
// DO NOT EDIT.

#include "tools/json_schema_compiler/util.h"
#include "base/logging.h"
#include "base/strings/string_number_conversions.h"
#include "base/strings/utf_string_conversions.h"
#include "base/values.h"
#include "chrome/common/extensions/api/system_private.h"
#include <set>
#include <utility>

using base::UTF8ToUTF16;

namespace extensions {
namespace api {
namespace system_private {
//
// Types
//

const char* ToString(UpdateStatusState enum_param) {
  switch (enum_param) {
    case UPDATE_STATUS_STATE_NOTAVAILABLE:
      return "NotAvailable";
    case UPDATE_STATUS_STATE_UPDATING:
      return "Updating";
    case UPDATE_STATUS_STATE_NEEDRESTART:
      return "NeedRestart";
    case UPDATE_STATUS_STATE_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

UpdateStatusState ParseUpdateStatusState(const std::string& enum_string) {
  if (enum_string == "NotAvailable")
    return UPDATE_STATUS_STATE_NOTAVAILABLE;
  if (enum_string == "Updating")
    return UPDATE_STATUS_STATE_UPDATING;
  if (enum_string == "NeedRestart")
    return UPDATE_STATUS_STATE_NEEDRESTART;
  return UPDATE_STATUS_STATE_NONE;
}


const char* ToString(GetIncognitoModeAvailabilityValue enum_param) {
  switch (enum_param) {
    case GET_INCOGNITO_MODE_AVAILABILITY_VALUE_ENABLED:
      return "enabled";
    case GET_INCOGNITO_MODE_AVAILABILITY_VALUE_DISABLED:
      return "disabled";
    case GET_INCOGNITO_MODE_AVAILABILITY_VALUE_FORCED:
      return "forced";
    case GET_INCOGNITO_MODE_AVAILABILITY_VALUE_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

GetIncognitoModeAvailabilityValue ParseGetIncognitoModeAvailabilityValue(const std::string& enum_string) {
  if (enum_string == "enabled")
    return GET_INCOGNITO_MODE_AVAILABILITY_VALUE_ENABLED;
  if (enum_string == "disabled")
    return GET_INCOGNITO_MODE_AVAILABILITY_VALUE_DISABLED;
  if (enum_string == "forced")
    return GET_INCOGNITO_MODE_AVAILABILITY_VALUE_FORCED;
  return GET_INCOGNITO_MODE_AVAILABILITY_VALUE_NONE;
}


UpdateStatus::UpdateStatus()
: state(UPDATE_STATUS_STATE_NONE),
download_progress(0.0) {}

UpdateStatus::~UpdateStatus() {}
UpdateStatus::UpdateStatus(UpdateStatus&& rhs)
: state(rhs.state),
download_progress(rhs.download_progress){
}

UpdateStatus& UpdateStatus::operator=(UpdateStatus&& rhs)
{
state = rhs.state;
download_progress = rhs.download_progress;
return *this;
}

// static
bool UpdateStatus::Populate(
    const base::Value& value, UpdateStatus* out) {
  if (!value.is_dict()) {
    return false;
  }
  const base::DictionaryValue* dict = static_cast<const base::DictionaryValue*>(&value);
  const base::Value* state_value = NULL;
  if (!dict->GetWithoutPathExpansion("state", &state_value)) {
    return false;
  }
  {
    std::string update_status_state_as_string;
    if (!state_value->GetAsString(&update_status_state_as_string)) {
      return false;
    }
    out->state = ParseUpdateStatusState(update_status_state_as_string);
    if (out->state == UPDATE_STATUS_STATE_NONE) {
      return false;
    }
  }

  const base::Value* download_progress_value = NULL;
  if (!dict->GetWithoutPathExpansion("downloadProgress", &download_progress_value)) {
    return false;
  }
  {
    if (!download_progress_value->GetAsDouble(&out->download_progress)) {
      return false;
    }
  }

  return true;
}

// static
std::unique_ptr<UpdateStatus> UpdateStatus::FromValue(const base::Value& value) {
  std::unique_ptr<UpdateStatus> out(new UpdateStatus());
  if (!Populate(value, out.get()))
    return nullptr;
  return out;
}

std::unique_ptr<base::DictionaryValue> UpdateStatus::ToValue() const {
  std::unique_ptr<base::DictionaryValue> value(new base::DictionaryValue());

  value->SetWithoutPathExpansion("state", std::make_unique<base::Value>(system_private::ToString(this->state)));

  value->SetWithoutPathExpansion("downloadProgress", std::make_unique<base::Value>(this->download_progress));


  return value;
}


VolumeInfo::VolumeInfo()
: volume(0.0),
is_volume_muted(false) {}

VolumeInfo::~VolumeInfo() {}
VolumeInfo::VolumeInfo(VolumeInfo&& rhs)
: volume(rhs.volume),
is_volume_muted(rhs.is_volume_muted){
}

VolumeInfo& VolumeInfo::operator=(VolumeInfo&& rhs)
{
volume = rhs.volume;
is_volume_muted = rhs.is_volume_muted;
return *this;
}

// static
bool VolumeInfo::Populate(
    const base::Value& value, VolumeInfo* out) {
  if (!value.is_dict()) {
    return false;
  }
  const base::DictionaryValue* dict = static_cast<const base::DictionaryValue*>(&value);
  const base::Value* volume_value = NULL;
  if (!dict->GetWithoutPathExpansion("volume", &volume_value)) {
    return false;
  }
  {
    if (!volume_value->GetAsDouble(&out->volume)) {
      return false;
    }
  }

  const base::Value* is_volume_muted_value = NULL;
  if (!dict->GetWithoutPathExpansion("isVolumeMuted", &is_volume_muted_value)) {
    return false;
  }
  {
    if (!is_volume_muted_value->GetAsBoolean(&out->is_volume_muted)) {
      return false;
    }
  }

  return true;
}

// static
std::unique_ptr<VolumeInfo> VolumeInfo::FromValue(const base::Value& value) {
  std::unique_ptr<VolumeInfo> out(new VolumeInfo());
  if (!Populate(value, out.get()))
    return nullptr;
  return out;
}

std::unique_ptr<base::DictionaryValue> VolumeInfo::ToValue() const {
  std::unique_ptr<base::DictionaryValue> value(new base::DictionaryValue());

  value->SetWithoutPathExpansion("volume", std::make_unique<base::Value>(this->volume));

  value->SetWithoutPathExpansion("isVolumeMuted", std::make_unique<base::Value>(this->is_volume_muted));


  return value;
}


BrightnessChangeInfo::BrightnessChangeInfo()
: brightness(0.0),
user_initiated(false) {}

BrightnessChangeInfo::~BrightnessChangeInfo() {}
BrightnessChangeInfo::BrightnessChangeInfo(BrightnessChangeInfo&& rhs)
: brightness(rhs.brightness),
user_initiated(rhs.user_initiated){
}

BrightnessChangeInfo& BrightnessChangeInfo::operator=(BrightnessChangeInfo&& rhs)
{
brightness = rhs.brightness;
user_initiated = rhs.user_initiated;
return *this;
}

// static
bool BrightnessChangeInfo::Populate(
    const base::Value& value, BrightnessChangeInfo* out) {
  if (!value.is_dict()) {
    return false;
  }
  const base::DictionaryValue* dict = static_cast<const base::DictionaryValue*>(&value);
  const base::Value* brightness_value = NULL;
  if (!dict->GetWithoutPathExpansion("brightness", &brightness_value)) {
    return false;
  }
  {
    if (!brightness_value->GetAsDouble(&out->brightness)) {
      return false;
    }
  }

  const base::Value* user_initiated_value = NULL;
  if (!dict->GetWithoutPathExpansion("userInitiated", &user_initiated_value)) {
    return false;
  }
  {
    if (!user_initiated_value->GetAsBoolean(&out->user_initiated)) {
      return false;
    }
  }

  return true;
}

// static
std::unique_ptr<BrightnessChangeInfo> BrightnessChangeInfo::FromValue(const base::Value& value) {
  std::unique_ptr<BrightnessChangeInfo> out(new BrightnessChangeInfo());
  if (!Populate(value, out.get()))
    return nullptr;
  return out;
}

std::unique_ptr<base::DictionaryValue> BrightnessChangeInfo::ToValue() const {
  std::unique_ptr<base::DictionaryValue> value(new base::DictionaryValue());

  value->SetWithoutPathExpansion("brightness", std::make_unique<base::Value>(this->brightness));

  value->SetWithoutPathExpansion("userInitiated", std::make_unique<base::Value>(this->user_initiated));


  return value;
}



//
// Functions
//

namespace GetIncognitoModeAvailability {

std::unique_ptr<base::ListValue> Results::Create(const GetIncognitoModeAvailabilityValue& value) {
  std::unique_ptr<base::ListValue> create_results(new base::ListValue());
  create_results->Append(std::make_unique<base::Value>(system_private::ToString(value)));

  return create_results;
}
}  // namespace GetIncognitoModeAvailability

namespace GetUpdateStatus {

std::unique_ptr<base::ListValue> Results::Create(const UpdateStatus& status) {
  std::unique_ptr<base::ListValue> create_results(new base::ListValue());
  create_results->Append((status).ToValue());

  return create_results;
}
}  // namespace GetUpdateStatus

namespace GetApiKey {

std::unique_ptr<base::ListValue> Results::Create(const std::string& key) {
  std::unique_ptr<base::ListValue> create_results(new base::ListValue());
  create_results->Append(std::make_unique<base::Value>(key));

  return create_results;
}
}  // namespace GetApiKey

//
// Events
//

namespace OnVolumeChanged {

const char kEventName[] = "systemPrivate.onVolumeChanged";

std::unique_ptr<base::ListValue> Create(const VolumeInfo& volume) {
  std::unique_ptr<base::ListValue> create_results(new base::ListValue());
  create_results->Append((volume).ToValue());

  return create_results;
}

}  // namespace OnVolumeChanged

namespace OnBrightnessChanged {

const char kEventName[] = "systemPrivate.onBrightnessChanged";

std::unique_ptr<base::ListValue> Create(const BrightnessChangeInfo& brightness) {
  std::unique_ptr<base::ListValue> create_results(new base::ListValue());
  create_results->Append((brightness).ToValue());

  return create_results;
}

}  // namespace OnBrightnessChanged

namespace OnScreenUnlocked {

const char kEventName[] = "systemPrivate.onScreenUnlocked";

std::unique_ptr<base::ListValue> Create() {
  std::unique_ptr<base::ListValue> create_results(new base::ListValue());
  return create_results;
}

}  // namespace OnScreenUnlocked

namespace OnWokeUp {

const char kEventName[] = "systemPrivate.onWokeUp";

std::unique_ptr<base::ListValue> Create() {
  std::unique_ptr<base::ListValue> create_results(new base::ListValue());
  return create_results;
}

}  // namespace OnWokeUp

}  // namespace system_private
}  // namespace api
}  // namespace extensions

