// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "content/public/common/resource_load_info.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "base/run_loop.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"

#include "content/public/common/resource_load_info.mojom-shared-message-ids.h"
#include "content/public/common/resource_type_struct_traits.h"
#include "mojo/public/cpp/base/time_mojom_traits.h"
#include "net/interfaces/ip_address_struct_traits.h"
#include "url/mojom/url_gurl_mojom_traits.h"
namespace content {
namespace mojom {
ResourceLoadInfo::ResourceLoadInfo()
    : request_id(),
      url(),
      referrer(),
      original_url(),
      method(),
      resource_type(),
      mime_type(),
      ip(),
      was_cached(),
      network_accessed(),
      net_error(),
      request_start() {}

ResourceLoadInfo::ResourceLoadInfo(
    int64_t request_id_in,
    const GURL& url_in,
    const GURL& referrer_in,
    const GURL& original_url_in,
    const std::string& method_in,
    content::ResourceType resource_type_in,
    const std::string& mime_type_in,
    const base::Optional<net::IPAddress>& ip_in,
    bool was_cached_in,
    bool network_accessed_in,
    int32_t net_error_in,
    base::TimeTicks request_start_in)
    : request_id(std::move(request_id_in)),
      url(std::move(url_in)),
      referrer(std::move(referrer_in)),
      original_url(std::move(original_url_in)),
      method(std::move(method_in)),
      resource_type(std::move(resource_type_in)),
      mime_type(std::move(mime_type_in)),
      ip(std::move(ip_in)),
      was_cached(std::move(was_cached_in)),
      network_accessed(std::move(network_accessed_in)),
      net_error(std::move(net_error_in)),
      request_start(std::move(request_start_in)) {}

ResourceLoadInfo::~ResourceLoadInfo() = default;

bool ResourceLoadInfo::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
}  // namespace mojom
}  // namespace content

namespace mojo {


// static
bool StructTraits<::content::mojom::ResourceLoadInfo::DataView, ::content::mojom::ResourceLoadInfoPtr>::Read(
    ::content::mojom::ResourceLoadInfo::DataView input,
    ::content::mojom::ResourceLoadInfoPtr* output) {
  bool success = true;
  ::content::mojom::ResourceLoadInfoPtr result(::content::mojom::ResourceLoadInfo::New());
  
      result->request_id = input.request_id();
      if (!input.ReadUrl(&result->url))
        success = false;
      if (!input.ReadReferrer(&result->referrer))
        success = false;
      if (!input.ReadOriginalUrl(&result->original_url))
        success = false;
      if (!input.ReadMethod(&result->method))
        success = false;
      if (!input.ReadResourceType(&result->resource_type))
        success = false;
      if (!input.ReadMimeType(&result->mime_type))
        success = false;
      if (!input.ReadIp(&result->ip))
        success = false;
      result->was_cached = input.was_cached();
      result->network_accessed = input.network_accessed();
      result->net_error = input.net_error();
      if (!input.ReadRequestStart(&result->request_start))
        success = false;
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif