// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_PUBLIC_COMMON_RESOURCE_LOAD_INFO_MOJOM_H_
#define CONTENT_PUBLIC_COMMON_RESOURCE_LOAD_INFO_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "content/public/common/resource_load_info.mojom-shared.h"
#include "content/public/common/resource_type.mojom.h"
#include "mojo/public/mojom/base/time.mojom.h"
#include "net/interfaces/ip_address.mojom.h"
#include "url/mojom/url.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "content/common/content_export.h"


namespace content {
namespace mojom {
class ResourceLoadInfo;
using ResourceLoadInfoPtr = mojo::StructPtr<ResourceLoadInfo>;






class CONTENT_EXPORT ResourceLoadInfo {
 public:
  using DataView = ResourceLoadInfoDataView;
  using Data_ = internal::ResourceLoadInfo_Data;

  template <typename... Args>
  static ResourceLoadInfoPtr New(Args&&... args) {
    return ResourceLoadInfoPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static ResourceLoadInfoPtr From(const U& u) {
    return mojo::TypeConverter<ResourceLoadInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ResourceLoadInfo>::Convert(*this);
  }


  ResourceLoadInfo();

  ResourceLoadInfo(
      int64_t request_id,
      const GURL& url,
      const GURL& referrer,
      const GURL& original_url,
      const std::string& method,
      content::ResourceType resource_type,
      const std::string& mime_type,
      const base::Optional<net::IPAddress>& ip,
      bool was_cached,
      bool network_accessed,
      int32_t net_error,
      base::TimeTicks request_start);

  ~ResourceLoadInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ResourceLoadInfoPtr>
  ResourceLoadInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ResourceLoadInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        ResourceLoadInfo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        ResourceLoadInfo::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::ResourceLoadInfo_UnserializedMessageContext<
            UserType, ResourceLoadInfo::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<ResourceLoadInfo::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return ResourceLoadInfo::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::ResourceLoadInfo_UnserializedMessageContext<
            UserType, ResourceLoadInfo::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<ResourceLoadInfo::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  int64_t request_id;
  GURL url;
  GURL referrer;
  GURL original_url;
  std::string method;
  content::ResourceType resource_type;
  std::string mime_type;
  base::Optional<net::IPAddress> ip;
  bool was_cached;
  bool network_accessed;
  int32_t net_error;
  base::TimeTicks request_start;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};

template <typename StructPtrType>
ResourceLoadInfoPtr ResourceLoadInfo::Clone() const {
  return New(
      mojo::Clone(request_id),
      mojo::Clone(url),
      mojo::Clone(referrer),
      mojo::Clone(original_url),
      mojo::Clone(method),
      mojo::Clone(resource_type),
      mojo::Clone(mime_type),
      mojo::Clone(ip),
      mojo::Clone(was_cached),
      mojo::Clone(network_accessed),
      mojo::Clone(net_error),
      mojo::Clone(request_start)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ResourceLoadInfo>::value>::type*>
bool ResourceLoadInfo::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->request_id, other_struct.request_id))
    return false;
  if (!mojo::Equals(this->url, other_struct.url))
    return false;
  if (!mojo::Equals(this->referrer, other_struct.referrer))
    return false;
  if (!mojo::Equals(this->original_url, other_struct.original_url))
    return false;
  if (!mojo::Equals(this->method, other_struct.method))
    return false;
  if (!mojo::Equals(this->resource_type, other_struct.resource_type))
    return false;
  if (!mojo::Equals(this->mime_type, other_struct.mime_type))
    return false;
  if (!mojo::Equals(this->ip, other_struct.ip))
    return false;
  if (!mojo::Equals(this->was_cached, other_struct.was_cached))
    return false;
  if (!mojo::Equals(this->network_accessed, other_struct.network_accessed))
    return false;
  if (!mojo::Equals(this->net_error, other_struct.net_error))
    return false;
  if (!mojo::Equals(this->request_start, other_struct.request_start))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace content

namespace mojo {


template <>
struct CONTENT_EXPORT StructTraits<::content::mojom::ResourceLoadInfo::DataView,
                                         ::content::mojom::ResourceLoadInfoPtr> {
  static bool IsNull(const ::content::mojom::ResourceLoadInfoPtr& input) { return !input; }
  static void SetToNull(::content::mojom::ResourceLoadInfoPtr* output) { output->reset(); }

  static decltype(::content::mojom::ResourceLoadInfo::request_id) request_id(
      const ::content::mojom::ResourceLoadInfoPtr& input) {
    return input->request_id;
  }

  static const decltype(::content::mojom::ResourceLoadInfo::url)& url(
      const ::content::mojom::ResourceLoadInfoPtr& input) {
    return input->url;
  }

  static const decltype(::content::mojom::ResourceLoadInfo::referrer)& referrer(
      const ::content::mojom::ResourceLoadInfoPtr& input) {
    return input->referrer;
  }

  static const decltype(::content::mojom::ResourceLoadInfo::original_url)& original_url(
      const ::content::mojom::ResourceLoadInfoPtr& input) {
    return input->original_url;
  }

  static const decltype(::content::mojom::ResourceLoadInfo::method)& method(
      const ::content::mojom::ResourceLoadInfoPtr& input) {
    return input->method;
  }

  static decltype(::content::mojom::ResourceLoadInfo::resource_type) resource_type(
      const ::content::mojom::ResourceLoadInfoPtr& input) {
    return input->resource_type;
  }

  static const decltype(::content::mojom::ResourceLoadInfo::mime_type)& mime_type(
      const ::content::mojom::ResourceLoadInfoPtr& input) {
    return input->mime_type;
  }

  static const decltype(::content::mojom::ResourceLoadInfo::ip)& ip(
      const ::content::mojom::ResourceLoadInfoPtr& input) {
    return input->ip;
  }

  static decltype(::content::mojom::ResourceLoadInfo::was_cached) was_cached(
      const ::content::mojom::ResourceLoadInfoPtr& input) {
    return input->was_cached;
  }

  static decltype(::content::mojom::ResourceLoadInfo::network_accessed) network_accessed(
      const ::content::mojom::ResourceLoadInfoPtr& input) {
    return input->network_accessed;
  }

  static decltype(::content::mojom::ResourceLoadInfo::net_error) net_error(
      const ::content::mojom::ResourceLoadInfoPtr& input) {
    return input->net_error;
  }

  static const decltype(::content::mojom::ResourceLoadInfo::request_start)& request_start(
      const ::content::mojom::ResourceLoadInfoPtr& input) {
    return input->request_start;
  }

  static bool Read(::content::mojom::ResourceLoadInfo::DataView input, ::content::mojom::ResourceLoadInfoPtr* output);
};

}  // namespace mojo

#endif  // CONTENT_PUBLIC_COMMON_RESOURCE_LOAD_INFO_MOJOM_H_