// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef HEADLESS_LIB_HEADLESS_RENDER_FRAME_CONTROLLER_MOJOM_H_
#define HEADLESS_LIB_HEADLESS_RENDER_FRAME_CONTROLLER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "headless/lib/headless_render_frame_controller.mojom-shared.h"
#include "mojo/public/mojom/base/big_string.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace headless {
class HeadlessRenderFrameController;
using HeadlessRenderFrameControllerPtr = mojo::InterfacePtr<HeadlessRenderFrameController>;
using HeadlessRenderFrameControllerPtrInfo = mojo::InterfacePtrInfo<HeadlessRenderFrameController>;
using ThreadSafeHeadlessRenderFrameControllerPtr =
    mojo::ThreadSafeInterfacePtr<HeadlessRenderFrameController>;
using HeadlessRenderFrameControllerRequest = mojo::InterfaceRequest<HeadlessRenderFrameController>;
using HeadlessRenderFrameControllerAssociatedPtr =
    mojo::AssociatedInterfacePtr<HeadlessRenderFrameController>;
using ThreadSafeHeadlessRenderFrameControllerAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<HeadlessRenderFrameController>;
using HeadlessRenderFrameControllerAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<HeadlessRenderFrameController>;
using HeadlessRenderFrameControllerAssociatedRequest =
    mojo::AssociatedInterfaceRequest<HeadlessRenderFrameController>;


class HeadlessRenderFrameControllerProxy;

template <typename ImplRefTraits>
class HeadlessRenderFrameControllerStub;

class HeadlessRenderFrameControllerRequestValidator;
class HeadlessRenderFrameControllerResponseValidator;

class  HeadlessRenderFrameController
    : public HeadlessRenderFrameControllerInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = HeadlessRenderFrameControllerProxy;

  template <typename ImplRefTraits>
  using Stub_ = HeadlessRenderFrameControllerStub<ImplRefTraits>;

  using RequestValidator_ = HeadlessRenderFrameControllerRequestValidator;
  using ResponseValidator_ = HeadlessRenderFrameControllerResponseValidator;
  enum MethodMinVersions : uint32_t {
    kInstallTabSocketMinVersion = 0,
    kInstallMainWorldTabSocketMinVersion = 0,
    kSendMessageToTabSocketMinVersion = 0,
  };
  virtual ~HeadlessRenderFrameController() {}


  using InstallTabSocketCallback = base::OnceCallback<void(bool)>;
  virtual void InstallTabSocket(int32_t v8_execution_context_id, InstallTabSocketCallback callback) = 0;


  using InstallMainWorldTabSocketCallback = base::OnceCallback<void(int32_t)>;
  virtual void InstallMainWorldTabSocket(InstallMainWorldTabSocketCallback callback) = 0;

  virtual void SendMessageToTabSocket(const std::string& message, int32_t v8_execution_context_id) = 0;
};
class  HeadlessRenderFrameControllerInterceptorForTesting : public HeadlessRenderFrameController {
  virtual HeadlessRenderFrameController* GetForwardingInterface() = 0;
  void InstallTabSocket(int32_t v8_execution_context_id, InstallTabSocketCallback callback) override;
  void InstallMainWorldTabSocket(InstallMainWorldTabSocketCallback callback) override;
  void SendMessageToTabSocket(const std::string& message, int32_t v8_execution_context_id) override;
};
class  HeadlessRenderFrameControllerAsyncWaiter {
 public:
  explicit HeadlessRenderFrameControllerAsyncWaiter(HeadlessRenderFrameController* proxy);
  ~HeadlessRenderFrameControllerAsyncWaiter();
  void InstallTabSocket(
      int32_t v8_execution_context_id, bool* out_success);
  void InstallMainWorldTabSocket(
      int32_t* out_v8_execution_context_id);

 private:
  HeadlessRenderFrameController* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(HeadlessRenderFrameControllerAsyncWaiter);
};

class  HeadlessRenderFrameControllerProxy
    : public HeadlessRenderFrameController {
 public:
  explicit HeadlessRenderFrameControllerProxy(mojo::MessageReceiverWithResponder* receiver);
  void InstallTabSocket(int32_t v8_execution_context_id, InstallTabSocketCallback callback) final;
  void InstallMainWorldTabSocket(InstallMainWorldTabSocketCallback callback) final;
  void SendMessageToTabSocket(const std::string& message, int32_t v8_execution_context_id) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  HeadlessRenderFrameControllerStubDispatch {
 public:
  static bool Accept(HeadlessRenderFrameController* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      HeadlessRenderFrameController* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<HeadlessRenderFrameController>>
class HeadlessRenderFrameControllerStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  HeadlessRenderFrameControllerStub() {}
  ~HeadlessRenderFrameControllerStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return HeadlessRenderFrameControllerStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return HeadlessRenderFrameControllerStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  HeadlessRenderFrameControllerRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  HeadlessRenderFrameControllerResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace headless

namespace mojo {

}  // namespace mojo

#endif  // HEADLESS_LIB_HEADLESS_RENDER_FRAME_CONTROLLER_MOJOM_H_