// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "headless/lib/tab_socket.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "base/run_loop.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"

#include "headless/lib/tab_socket.mojom-shared-message-ids.h"
#include "mojo/public/cpp/base/big_buffer_mojom_traits.h"
#include "mojo/public/cpp/base/big_string_mojom_traits.h"
namespace headless {
const char TabSocket::Name_[] = "headless::TabSocket";

TabSocketProxy::TabSocketProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void TabSocketProxy::SendMessageToEmbedder(
    const std::string& in_message, int32_t in_v8_execution_context_id) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "headless::TabSocket::SendMessageToEmbedder");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kTabSocket_SendMessageToEmbedder_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::headless::internal::TabSocket_SendMessageToEmbedder_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->message)::BaseType::BufferWriter
      message_writer;
  mojo::internal::Serialize<::mojo_base::mojom::BigStringDataView>(
      in_message, buffer, &message_writer, &serialization_context);
  params->message.Set(
      message_writer.is_null() ? nullptr : message_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->message.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null message in TabSocket.SendMessageToEmbedder request");
  params->v8_execution_context_id = in_v8_execution_context_id;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool TabSocketStubDispatch::Accept(
    TabSocket* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kTabSocket_SendMessageToEmbedder_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)headless::TabSocket::SendMessageToEmbedder",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::TabSocket_SendMessageToEmbedder_Params_Data* params =
          reinterpret_cast<internal::TabSocket_SendMessageToEmbedder_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_message{};
      int32_t p_v8_execution_context_id{};
      TabSocket_SendMessageToEmbedder_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadMessage(&p_message))
        success = false;
      p_v8_execution_context_id = input_data_view.v8_execution_context_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "TabSocket::SendMessageToEmbedder deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SendMessageToEmbedder(
std::move(p_message), 
std::move(p_v8_execution_context_id));
      return true;
    }
  }
  return false;
}

// static
bool TabSocketStubDispatch::AcceptWithResponder(
    TabSocket* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kTabSocket_SendMessageToEmbedder_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)headless::TabSocket::SendMessageToEmbedder",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool TabSocketRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "TabSocket RequestValidator");

  switch (message->header()->name) {
    case internal::kTabSocket_SendMessageToEmbedder_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::TabSocket_SendMessageToEmbedder_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void TabSocketInterceptorForTesting::SendMessageToEmbedder(const std::string& message, int32_t v8_execution_context_id) {
  GetForwardingInterface()->SendMessageToEmbedder(std::move(message), std::move(v8_execution_context_id));
}
TabSocketAsyncWaiter::TabSocketAsyncWaiter(
    TabSocket* proxy) : proxy_(proxy) {}

TabSocketAsyncWaiter::~TabSocketAsyncWaiter() = default;


}  // namespace headless

namespace mojo {

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif