// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef HEADLESS_LIB_TAB_SOCKET_MOJOM_H_
#define HEADLESS_LIB_TAB_SOCKET_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "headless/lib/tab_socket.mojom-shared.h"
#include "mojo/public/mojom/base/big_string.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace headless {
class TabSocket;
using TabSocketPtr = mojo::InterfacePtr<TabSocket>;
using TabSocketPtrInfo = mojo::InterfacePtrInfo<TabSocket>;
using ThreadSafeTabSocketPtr =
    mojo::ThreadSafeInterfacePtr<TabSocket>;
using TabSocketRequest = mojo::InterfaceRequest<TabSocket>;
using TabSocketAssociatedPtr =
    mojo::AssociatedInterfacePtr<TabSocket>;
using ThreadSafeTabSocketAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<TabSocket>;
using TabSocketAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<TabSocket>;
using TabSocketAssociatedRequest =
    mojo::AssociatedInterfaceRequest<TabSocket>;


class TabSocketProxy;

template <typename ImplRefTraits>
class TabSocketStub;

class TabSocketRequestValidator;

class  TabSocket
    : public TabSocketInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = TabSocketProxy;

  template <typename ImplRefTraits>
  using Stub_ = TabSocketStub<ImplRefTraits>;

  using RequestValidator_ = TabSocketRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kSendMessageToEmbedderMinVersion = 0,
  };
  virtual ~TabSocket() {}

  virtual void SendMessageToEmbedder(const std::string& message, int32_t v8_execution_context_id) = 0;
};
class  TabSocketInterceptorForTesting : public TabSocket {
  virtual TabSocket* GetForwardingInterface() = 0;
  void SendMessageToEmbedder(const std::string& message, int32_t v8_execution_context_id) override;
};
class  TabSocketAsyncWaiter {
 public:
  explicit TabSocketAsyncWaiter(TabSocket* proxy);
  ~TabSocketAsyncWaiter();

 private:
  TabSocket* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(TabSocketAsyncWaiter);
};

class  TabSocketProxy
    : public TabSocket {
 public:
  explicit TabSocketProxy(mojo::MessageReceiverWithResponder* receiver);
  void SendMessageToEmbedder(const std::string& message, int32_t v8_execution_context_id) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  TabSocketStubDispatch {
 public:
  static bool Accept(TabSocket* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      TabSocket* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<TabSocket>>
class TabSocketStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  TabSocketStub() {}
  ~TabSocketStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return TabSocketStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return TabSocketStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  TabSocketRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace headless

namespace mojo {

}  // namespace mojo

#endif  // HEADLESS_LIB_TAB_SOCKET_MOJOM_H_