// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef MOJO_PUBLIC_MOJOM_BASE_SHARED_MEMORY_MOJOM_SHARED_H_
#define MOJO_PUBLIC_MOJOM_BASE_SHARED_MEMORY_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "mojo/public/mojom/base/shared_memory.mojom-shared-internal.h"
#include "mojo/public/mojom/base/unguessable_token.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "base/component_export.h"


namespace mojo_base {
namespace mojom {
class PlatformSharedMemoryHandleDataView;

class PlatformSharedMemoryRegionDataView;

class ReadOnlySharedMemoryRegionDataView;

class WritableSharedMemoryRegionDataView;

class UnsafeSharedMemoryRegionDataView;



}  // namespace mojom
}  // namespace mojo_base

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::mojo_base::mojom::PlatformSharedMemoryHandleDataView> {
  using Data = ::mojo_base::mojom::internal::PlatformSharedMemoryHandle_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::mojo_base::mojom::PlatformSharedMemoryRegionDataView> {
  using Data = ::mojo_base::mojom::internal::PlatformSharedMemoryRegion_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::mojo_base::mojom::ReadOnlySharedMemoryRegionDataView> {
  using Data = ::mojo_base::mojom::internal::ReadOnlySharedMemoryRegion_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::mojo_base::mojom::WritableSharedMemoryRegionDataView> {
  using Data = ::mojo_base::mojom::internal::WritableSharedMemoryRegion_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::mojo_base::mojom::UnsafeSharedMemoryRegionDataView> {
  using Data = ::mojo_base::mojom::internal::UnsafeSharedMemoryRegion_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace mojo_base {
namespace mojom {

enum class PlatformSharedMemoryRegion_Mode : int32_t {
  kReadOnly,
  kWritable,
  kUnsafe,
  kMinValue = 0,
  kMaxValue = 2,
};

inline std::ostream& operator<<(std::ostream& os, PlatformSharedMemoryRegion_Mode value) {
  switch(value) {
    case PlatformSharedMemoryRegion_Mode::kReadOnly:
      return os << "PlatformSharedMemoryRegion_Mode::kReadOnly";
    case PlatformSharedMemoryRegion_Mode::kWritable:
      return os << "PlatformSharedMemoryRegion_Mode::kWritable";
    case PlatformSharedMemoryRegion_Mode::kUnsafe:
      return os << "PlatformSharedMemoryRegion_Mode::kUnsafe";
    default:
      return os << "Unknown PlatformSharedMemoryRegion_Mode value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(PlatformSharedMemoryRegion_Mode value) {
  return internal::PlatformSharedMemoryRegion_Mode_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
class PlatformSharedMemoryHandleDataView {
 public:
  PlatformSharedMemoryHandleDataView() {}

  PlatformSharedMemoryHandleDataView(
      internal::PlatformSharedMemoryHandle_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  mojo::ScopedHandle TakeHandleValue() {
    mojo::ScopedHandle result;
    bool ret =
        mojo::internal::Deserialize<mojo::ScopedHandle>(
            &data_->handle_value, &result, context_);
    DCHECK(ret);
    return result;
  }
  mojo::ScopedHandle TakeReadonlyFd() {
    mojo::ScopedHandle result;
    bool ret =
        mojo::internal::Deserialize<mojo::ScopedHandle>(
            &data_->readonly_fd, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::PlatformSharedMemoryHandle_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PlatformSharedMemoryRegionDataView {
 public:
  PlatformSharedMemoryRegionDataView() {}

  PlatformSharedMemoryRegionDataView(
      internal::PlatformSharedMemoryRegion_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetPlatformHandleDataView(
      PlatformSharedMemoryHandleDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPlatformHandle(UserType* output) {
    auto* pointer = data_->platform_handle.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::PlatformSharedMemoryHandleDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMode(UserType* output) const {
    auto data_value = data_->mode;
    return mojo::internal::Deserialize<::mojo_base::mojom::PlatformSharedMemoryRegion_Mode>(
        data_value, output);
  }

  PlatformSharedMemoryRegion_Mode mode() const {
    return static_cast<PlatformSharedMemoryRegion_Mode>(data_->mode);
  }
  uint64_t size() const {
    return data_->size;
  }
  inline void GetGuidDataView(
      ::mojo_base::mojom::UnguessableTokenDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadGuid(UserType* output) {
    auto* pointer = data_->guid.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::UnguessableTokenDataView>(
        pointer, output, context_);
  }
 private:
  internal::PlatformSharedMemoryRegion_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ReadOnlySharedMemoryRegionDataView {
 public:
  ReadOnlySharedMemoryRegionDataView() {}

  ReadOnlySharedMemoryRegionDataView(
      internal::ReadOnlySharedMemoryRegion_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetRegionDataView(
      PlatformSharedMemoryRegionDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRegion(UserType* output) {
    auto* pointer = data_->region.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::PlatformSharedMemoryRegionDataView>(
        pointer, output, context_);
  }
 private:
  internal::ReadOnlySharedMemoryRegion_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class WritableSharedMemoryRegionDataView {
 public:
  WritableSharedMemoryRegionDataView() {}

  WritableSharedMemoryRegionDataView(
      internal::WritableSharedMemoryRegion_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetRegionDataView(
      PlatformSharedMemoryRegionDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRegion(UserType* output) {
    auto* pointer = data_->region.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::PlatformSharedMemoryRegionDataView>(
        pointer, output, context_);
  }
 private:
  internal::WritableSharedMemoryRegion_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class UnsafeSharedMemoryRegionDataView {
 public:
  UnsafeSharedMemoryRegionDataView() {}

  UnsafeSharedMemoryRegionDataView(
      internal::UnsafeSharedMemoryRegion_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetRegionDataView(
      PlatformSharedMemoryRegionDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRegion(UserType* output) {
    auto* pointer = data_->region.Get();
    return mojo::internal::Deserialize<::mojo_base::mojom::PlatformSharedMemoryRegionDataView>(
        pointer, output, context_);
  }
 private:
  internal::UnsafeSharedMemoryRegion_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace mojo_base

namespace std {

template <>
struct hash<::mojo_base::mojom::PlatformSharedMemoryRegion_Mode>
    : public mojo::internal::EnumHashImpl<::mojo_base::mojom::PlatformSharedMemoryRegion_Mode> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::mojo_base::mojom::PlatformSharedMemoryRegion_Mode, ::mojo_base::mojom::PlatformSharedMemoryRegion_Mode> {
  static ::mojo_base::mojom::PlatformSharedMemoryRegion_Mode ToMojom(::mojo_base::mojom::PlatformSharedMemoryRegion_Mode input) { return input; }
  static bool FromMojom(::mojo_base::mojom::PlatformSharedMemoryRegion_Mode input, ::mojo_base::mojom::PlatformSharedMemoryRegion_Mode* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::mojo_base::mojom::PlatformSharedMemoryRegion_Mode, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::mojo_base::mojom::PlatformSharedMemoryRegion_Mode, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::mojo_base::mojom::PlatformSharedMemoryRegion_Mode>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::mojo_base::mojom::PlatformSharedMemoryHandleDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::mojo_base::mojom::PlatformSharedMemoryHandleDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::mojo_base::mojom::internal::PlatformSharedMemoryHandle_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    decltype(CallWithContext(Traits::handle_value, input, custom_context)) in_handle_value = CallWithContext(Traits::handle_value, input, custom_context);
    mojo::internal::Serialize<mojo::ScopedHandle>(
        in_handle_value, &(*output)->handle_value, context);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        !mojo::internal::IsHandleOrInterfaceValid((*output)->handle_value),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
        "invalid handle_value in PlatformSharedMemoryHandle struct");
    decltype(CallWithContext(Traits::readonly_fd, input, custom_context)) in_readonly_fd = CallWithContext(Traits::readonly_fd, input, custom_context);
    mojo::internal::Serialize<mojo::ScopedHandle>(
        in_readonly_fd, &(*output)->readonly_fd, context);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::mojo_base::mojom::internal::PlatformSharedMemoryHandle_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::mojo_base::mojom::PlatformSharedMemoryHandleDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::mojo_base::mojom::PlatformSharedMemoryRegionDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::mojo_base::mojom::PlatformSharedMemoryRegionDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::mojo_base::mojom::internal::PlatformSharedMemoryRegion_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    decltype(CallWithContext(Traits::platform_handle, input, custom_context)) in_platform_handle = CallWithContext(Traits::platform_handle, input, custom_context);
    typename decltype((*output)->platform_handle)::BaseType::BufferWriter
        platform_handle_writer;
    mojo::internal::Serialize<::mojo_base::mojom::PlatformSharedMemoryHandleDataView>(
        in_platform_handle, buffer, &platform_handle_writer, context);
    (*output)->platform_handle.Set(
        platform_handle_writer.is_null() ? nullptr : platform_handle_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->platform_handle.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null platform_handle in PlatformSharedMemoryRegion struct");
    mojo::internal::Serialize<::mojo_base::mojom::PlatformSharedMemoryRegion_Mode>(
        CallWithContext(Traits::mode, input, custom_context), &(*output)->mode);
    (*output)->size = CallWithContext(Traits::size, input, custom_context);
    decltype(CallWithContext(Traits::guid, input, custom_context)) in_guid = CallWithContext(Traits::guid, input, custom_context);
    typename decltype((*output)->guid)::BaseType::BufferWriter
        guid_writer;
    mojo::internal::Serialize<::mojo_base::mojom::UnguessableTokenDataView>(
        in_guid, buffer, &guid_writer, context);
    (*output)->guid.Set(
        guid_writer.is_null() ? nullptr : guid_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->guid.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null guid in PlatformSharedMemoryRegion struct");
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::mojo_base::mojom::internal::PlatformSharedMemoryRegion_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::mojo_base::mojom::PlatformSharedMemoryRegionDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::mojo_base::mojom::ReadOnlySharedMemoryRegionDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::mojo_base::mojom::ReadOnlySharedMemoryRegionDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::mojo_base::mojom::internal::ReadOnlySharedMemoryRegion_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    decltype(CallWithContext(Traits::region, input, custom_context)) in_region = CallWithContext(Traits::region, input, custom_context);
    typename decltype((*output)->region)::BaseType::BufferWriter
        region_writer;
    mojo::internal::Serialize<::mojo_base::mojom::PlatformSharedMemoryRegionDataView>(
        in_region, buffer, &region_writer, context);
    (*output)->region.Set(
        region_writer.is_null() ? nullptr : region_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->region.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null region in ReadOnlySharedMemoryRegion struct");
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::mojo_base::mojom::internal::ReadOnlySharedMemoryRegion_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::mojo_base::mojom::ReadOnlySharedMemoryRegionDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::mojo_base::mojom::WritableSharedMemoryRegionDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::mojo_base::mojom::WritableSharedMemoryRegionDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::mojo_base::mojom::internal::WritableSharedMemoryRegion_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    decltype(CallWithContext(Traits::region, input, custom_context)) in_region = CallWithContext(Traits::region, input, custom_context);
    typename decltype((*output)->region)::BaseType::BufferWriter
        region_writer;
    mojo::internal::Serialize<::mojo_base::mojom::PlatformSharedMemoryRegionDataView>(
        in_region, buffer, &region_writer, context);
    (*output)->region.Set(
        region_writer.is_null() ? nullptr : region_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->region.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null region in WritableSharedMemoryRegion struct");
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::mojo_base::mojom::internal::WritableSharedMemoryRegion_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::mojo_base::mojom::WritableSharedMemoryRegionDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::mojo_base::mojom::UnsafeSharedMemoryRegionDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::mojo_base::mojom::UnsafeSharedMemoryRegionDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::mojo_base::mojom::internal::UnsafeSharedMemoryRegion_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    decltype(CallWithContext(Traits::region, input, custom_context)) in_region = CallWithContext(Traits::region, input, custom_context);
    typename decltype((*output)->region)::BaseType::BufferWriter
        region_writer;
    mojo::internal::Serialize<::mojo_base::mojom::PlatformSharedMemoryRegionDataView>(
        in_region, buffer, &region_writer, context);
    (*output)->region.Set(
        region_writer.is_null() ? nullptr : region_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->region.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null region in UnsafeSharedMemoryRegion struct");
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::mojo_base::mojom::internal::UnsafeSharedMemoryRegion_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::mojo_base::mojom::UnsafeSharedMemoryRegionDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace mojo_base {
namespace mojom {



inline void PlatformSharedMemoryRegionDataView::GetPlatformHandleDataView(
    PlatformSharedMemoryHandleDataView* output) {
  auto pointer = data_->platform_handle.Get();
  *output = PlatformSharedMemoryHandleDataView(pointer, context_);
}
inline void PlatformSharedMemoryRegionDataView::GetGuidDataView(
    ::mojo_base::mojom::UnguessableTokenDataView* output) {
  auto pointer = data_->guid.Get();
  *output = ::mojo_base::mojom::UnguessableTokenDataView(pointer, context_);
}


inline void ReadOnlySharedMemoryRegionDataView::GetRegionDataView(
    PlatformSharedMemoryRegionDataView* output) {
  auto pointer = data_->region.Get();
  *output = PlatformSharedMemoryRegionDataView(pointer, context_);
}


inline void WritableSharedMemoryRegionDataView::GetRegionDataView(
    PlatformSharedMemoryRegionDataView* output) {
  auto pointer = data_->region.Get();
  *output = PlatformSharedMemoryRegionDataView(pointer, context_);
}


inline void UnsafeSharedMemoryRegionDataView::GetRegionDataView(
    PlatformSharedMemoryRegionDataView* output) {
  auto pointer = data_->region.Get();
  *output = PlatformSharedMemoryRegionDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace mojo_base

#endif  // MOJO_PUBLIC_MOJOM_BASE_SHARED_MEMORY_MOJOM_SHARED_H_
