// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_NETWORK_PUBLIC_MOJOM_TCP_SOCKET_MOJOM_H_
#define SERVICES_NETWORK_PUBLIC_MOJOM_TCP_SOCKET_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/network/public/mojom/tcp_socket.mojom-shared.h"
#include "net/interfaces/ip_endpoint.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"


namespace network {
namespace mojom {
class TCPConnectedSocket;
using TCPConnectedSocketPtr = mojo::InterfacePtr<TCPConnectedSocket>;
using TCPConnectedSocketPtrInfo = mojo::InterfacePtrInfo<TCPConnectedSocket>;
using ThreadSafeTCPConnectedSocketPtr =
    mojo::ThreadSafeInterfacePtr<TCPConnectedSocket>;
using TCPConnectedSocketRequest = mojo::InterfaceRequest<TCPConnectedSocket>;
using TCPConnectedSocketAssociatedPtr =
    mojo::AssociatedInterfacePtr<TCPConnectedSocket>;
using ThreadSafeTCPConnectedSocketAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<TCPConnectedSocket>;
using TCPConnectedSocketAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<TCPConnectedSocket>;
using TCPConnectedSocketAssociatedRequest =
    mojo::AssociatedInterfaceRequest<TCPConnectedSocket>;

class TCPConnectedSocketObserver;
using TCPConnectedSocketObserverPtr = mojo::InterfacePtr<TCPConnectedSocketObserver>;
using TCPConnectedSocketObserverPtrInfo = mojo::InterfacePtrInfo<TCPConnectedSocketObserver>;
using ThreadSafeTCPConnectedSocketObserverPtr =
    mojo::ThreadSafeInterfacePtr<TCPConnectedSocketObserver>;
using TCPConnectedSocketObserverRequest = mojo::InterfaceRequest<TCPConnectedSocketObserver>;
using TCPConnectedSocketObserverAssociatedPtr =
    mojo::AssociatedInterfacePtr<TCPConnectedSocketObserver>;
using ThreadSafeTCPConnectedSocketObserverAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<TCPConnectedSocketObserver>;
using TCPConnectedSocketObserverAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<TCPConnectedSocketObserver>;
using TCPConnectedSocketObserverAssociatedRequest =
    mojo::AssociatedInterfaceRequest<TCPConnectedSocketObserver>;

class TCPServerSocket;
using TCPServerSocketPtr = mojo::InterfacePtr<TCPServerSocket>;
using TCPServerSocketPtrInfo = mojo::InterfacePtrInfo<TCPServerSocket>;
using ThreadSafeTCPServerSocketPtr =
    mojo::ThreadSafeInterfacePtr<TCPServerSocket>;
using TCPServerSocketRequest = mojo::InterfaceRequest<TCPServerSocket>;
using TCPServerSocketAssociatedPtr =
    mojo::AssociatedInterfacePtr<TCPServerSocket>;
using ThreadSafeTCPServerSocketAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<TCPServerSocket>;
using TCPServerSocketAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<TCPServerSocket>;
using TCPServerSocketAssociatedRequest =
    mojo::AssociatedInterfaceRequest<TCPServerSocket>;


class TCPConnectedSocketProxy;

template <typename ImplRefTraits>
class TCPConnectedSocketStub;

class TCPConnectedSocketRequestValidator;
class TCPConnectedSocketResponseValidator;

class  TCPConnectedSocket
    : public TCPConnectedSocketInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = TCPConnectedSocketProxy;

  template <typename ImplRefTraits>
  using Stub_ = TCPConnectedSocketStub<ImplRefTraits>;

  using RequestValidator_ = TCPConnectedSocketRequestValidator;
  using ResponseValidator_ = TCPConnectedSocketResponseValidator;
  enum MethodMinVersions : uint32_t {
    kGetLocalAddressMinVersion = 0,
  };
  virtual ~TCPConnectedSocket() {}


  using GetLocalAddressCallback = base::OnceCallback<void(int32_t, const base::Optional<net::IPEndPoint>&)>;
  virtual void GetLocalAddress(GetLocalAddressCallback callback) = 0;
};
class  TCPConnectedSocketInterceptorForTesting : public TCPConnectedSocket {
  virtual TCPConnectedSocket* GetForwardingInterface() = 0;
  void GetLocalAddress(GetLocalAddressCallback callback) override;
};
class  TCPConnectedSocketAsyncWaiter {
 public:
  explicit TCPConnectedSocketAsyncWaiter(TCPConnectedSocket* proxy);
  ~TCPConnectedSocketAsyncWaiter();
  void GetLocalAddress(
      int32_t* out_net_error, base::Optional<net::IPEndPoint>* out_local_addr);

 private:
  TCPConnectedSocket* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(TCPConnectedSocketAsyncWaiter);
};

class TCPConnectedSocketObserverProxy;

template <typename ImplRefTraits>
class TCPConnectedSocketObserverStub;

class TCPConnectedSocketObserverRequestValidator;

class  TCPConnectedSocketObserver
    : public TCPConnectedSocketObserverInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = TCPConnectedSocketObserverProxy;

  template <typename ImplRefTraits>
  using Stub_ = TCPConnectedSocketObserverStub<ImplRefTraits>;

  using RequestValidator_ = TCPConnectedSocketObserverRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kOnReadErrorMinVersion = 0,
    kOnWriteErrorMinVersion = 0,
  };
  virtual ~TCPConnectedSocketObserver() {}

  virtual void OnReadError(int32_t net_error) = 0;

  virtual void OnWriteError(int32_t net_error) = 0;
};
class  TCPConnectedSocketObserverInterceptorForTesting : public TCPConnectedSocketObserver {
  virtual TCPConnectedSocketObserver* GetForwardingInterface() = 0;
  void OnReadError(int32_t net_error) override;
  void OnWriteError(int32_t net_error) override;
};
class  TCPConnectedSocketObserverAsyncWaiter {
 public:
  explicit TCPConnectedSocketObserverAsyncWaiter(TCPConnectedSocketObserver* proxy);
  ~TCPConnectedSocketObserverAsyncWaiter();

 private:
  TCPConnectedSocketObserver* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(TCPConnectedSocketObserverAsyncWaiter);
};

class TCPServerSocketProxy;

template <typename ImplRefTraits>
class TCPServerSocketStub;

class TCPServerSocketRequestValidator;
class TCPServerSocketResponseValidator;

class  TCPServerSocket
    : public TCPServerSocketInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = TCPServerSocketProxy;

  template <typename ImplRefTraits>
  using Stub_ = TCPServerSocketStub<ImplRefTraits>;

  using RequestValidator_ = TCPServerSocketRequestValidator;
  using ResponseValidator_ = TCPServerSocketResponseValidator;
  enum MethodMinVersions : uint32_t {
    kAcceptMinVersion = 0,
  };
  virtual ~TCPServerSocket() {}


  using AcceptCallback = base::OnceCallback<void(int32_t, const base::Optional<net::IPEndPoint>&, TCPConnectedSocketPtr, mojo::ScopedDataPipeConsumerHandle, mojo::ScopedDataPipeProducerHandle)>;
  virtual void Accept(TCPConnectedSocketObserverPtr observer, AcceptCallback callback) = 0;
};
class  TCPServerSocketInterceptorForTesting : public TCPServerSocket {
  virtual TCPServerSocket* GetForwardingInterface() = 0;
  void Accept(TCPConnectedSocketObserverPtr observer, AcceptCallback callback) override;
};
class  TCPServerSocketAsyncWaiter {
 public:
  explicit TCPServerSocketAsyncWaiter(TCPServerSocket* proxy);
  ~TCPServerSocketAsyncWaiter();
  void Accept(
      TCPConnectedSocketObserverPtr observer, int32_t* out_net_error, base::Optional<net::IPEndPoint>* out_remote_addr, TCPConnectedSocketPtr* out_connected_socket, mojo::ScopedDataPipeConsumerHandle* out_send_stream, mojo::ScopedDataPipeProducerHandle* out_receive_stream);

 private:
  TCPServerSocket* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(TCPServerSocketAsyncWaiter);
};

class  TCPConnectedSocketProxy
    : public TCPConnectedSocket {
 public:
  explicit TCPConnectedSocketProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetLocalAddress(GetLocalAddressCallback callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  TCPConnectedSocketObserverProxy
    : public TCPConnectedSocketObserver {
 public:
  explicit TCPConnectedSocketObserverProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnReadError(int32_t net_error) final;
  void OnWriteError(int32_t net_error) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  TCPServerSocketProxy
    : public TCPServerSocket {
 public:
  explicit TCPServerSocketProxy(mojo::MessageReceiverWithResponder* receiver);
  void Accept(TCPConnectedSocketObserverPtr observer, AcceptCallback callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  TCPConnectedSocketStubDispatch {
 public:
  static bool Accept(TCPConnectedSocket* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      TCPConnectedSocket* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<TCPConnectedSocket>>
class TCPConnectedSocketStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  TCPConnectedSocketStub() {}
  ~TCPConnectedSocketStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return TCPConnectedSocketStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return TCPConnectedSocketStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  TCPConnectedSocketObserverStubDispatch {
 public:
  static bool Accept(TCPConnectedSocketObserver* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      TCPConnectedSocketObserver* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<TCPConnectedSocketObserver>>
class TCPConnectedSocketObserverStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  TCPConnectedSocketObserverStub() {}
  ~TCPConnectedSocketObserverStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return TCPConnectedSocketObserverStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return TCPConnectedSocketObserverStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  TCPServerSocketStubDispatch {
 public:
  static bool Accept(TCPServerSocket* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      TCPServerSocket* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<TCPServerSocket>>
class TCPServerSocketStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  TCPServerSocketStub() {}
  ~TCPServerSocketStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return TCPServerSocketStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return TCPServerSocketStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  TCPConnectedSocketRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  TCPConnectedSocketObserverRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  TCPServerSocketRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  TCPConnectedSocketResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  TCPServerSocketResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace network

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_NETWORK_PUBLIC_MOJOM_TCP_SOCKET_MOJOM_H_