// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "services/network/public/mojom/network_service.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "base/run_loop.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"

#include "services/network/public/mojom/network_service.mojom-shared-message-ids.h"
#include "ipc/ipc_message_utils.h"
#include "mojo/public/cpp/base/time_mojom_traits.h"
#include "net/interfaces/address_list_mojom_traits.h"
#include "net/interfaces/ip_address_struct_traits.h"
#include "net/interfaces/ip_endpoint_struct_traits.h"
#include "services/network/public/cpp/mutable_network_traffic_annotation_tag_mojom_traits.h"
#include "services/network/public/cpp/network_param_ipc_traits.h"
#include "services/network/public/cpp/proxy_config_mojom_traits.h"
#include "services/network/public/cpp/proxy_config_with_annotation_mojom_traits.h"
#include "url/mojom/origin_mojom_traits.h"
#include "url/mojom/url_gurl_mojom_traits.h"
namespace network {
namespace mojom {
NetworkContextParams::NetworkContextParams()
    : context_name(),
      user_agent(),
      accept_language(),
      enable_brotli(true),
      quic_user_agent_id(),
      proxy_resolver_factory(),
      cookie_path(),
      restore_old_session_cookies(false),
      persist_session_cookies(false),
      channel_id_path(),
      http_cache_enabled(true),
      http_cache_max_size(0),
      http_cache_path(),
      http_server_properties_path(),
      enable_data_url_support(false),
      enable_file_url_support(false),
      enable_ftp_url_support(false),
      http_09_on_non_default_ports_enabled(false),
      initial_proxy_config(),
      proxy_config_client_request(),
      proxy_config_poller_client(),
      allow_gssapi_library_load(false),
      gssapi_library_name() {}

NetworkContextParams::NetworkContextParams(
    const base::Optional<std::string>& context_name_in,
    const std::string& user_agent_in,
    const base::Optional<std::string>& accept_language_in,
    bool enable_brotli_in,
    const std::string& quic_user_agent_id_in,
    ::proxy_resolver::mojom::ProxyResolverFactoryPtrInfo proxy_resolver_factory_in,
    const base::Optional<base::FilePath>& cookie_path_in,
    bool restore_old_session_cookies_in,
    bool persist_session_cookies_in,
    const base::Optional<base::FilePath>& channel_id_path_in,
    bool http_cache_enabled_in,
    int32_t http_cache_max_size_in,
    const base::Optional<base::FilePath>& http_cache_path_in,
    const base::Optional<base::FilePath>& http_server_properties_path_in,
    bool enable_data_url_support_in,
    bool enable_file_url_support_in,
    bool enable_ftp_url_support_in,
    bool http_09_on_non_default_ports_enabled_in,
    const base::Optional<net::ProxyConfigWithAnnotation>& initial_proxy_config_in,
    ::network::mojom::ProxyConfigClientRequest proxy_config_client_request_in,
    ::network::mojom::ProxyConfigPollerClientPtrInfo proxy_config_poller_client_in,
    bool allow_gssapi_library_load_in,
    const std::string& gssapi_library_name_in)
    : context_name(std::move(context_name_in)),
      user_agent(std::move(user_agent_in)),
      accept_language(std::move(accept_language_in)),
      enable_brotli(std::move(enable_brotli_in)),
      quic_user_agent_id(std::move(quic_user_agent_id_in)),
      proxy_resolver_factory(std::move(proxy_resolver_factory_in)),
      cookie_path(std::move(cookie_path_in)),
      restore_old_session_cookies(std::move(restore_old_session_cookies_in)),
      persist_session_cookies(std::move(persist_session_cookies_in)),
      channel_id_path(std::move(channel_id_path_in)),
      http_cache_enabled(std::move(http_cache_enabled_in)),
      http_cache_max_size(std::move(http_cache_max_size_in)),
      http_cache_path(std::move(http_cache_path_in)),
      http_server_properties_path(std::move(http_server_properties_path_in)),
      enable_data_url_support(std::move(enable_data_url_support_in)),
      enable_file_url_support(std::move(enable_file_url_support_in)),
      enable_ftp_url_support(std::move(enable_ftp_url_support_in)),
      http_09_on_non_default_ports_enabled(std::move(http_09_on_non_default_ports_enabled_in)),
      initial_proxy_config(std::move(initial_proxy_config_in)),
      proxy_config_client_request(std::move(proxy_config_client_request_in)),
      proxy_config_poller_client(std::move(proxy_config_poller_client_in)),
      allow_gssapi_library_load(std::move(allow_gssapi_library_load_in)),
      gssapi_library_name(std::move(gssapi_library_name_in)) {}

NetworkContextParams::~NetworkContextParams() = default;

bool NetworkContextParams::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
NetworkConditions::NetworkConditions()
    : offline(),
      latency(),
      download_throughput(),
      upload_throughput() {}

NetworkConditions::NetworkConditions(
    bool offline_in,
    base::TimeDelta latency_in,
    double download_throughput_in,
    double upload_throughput_in)
    : offline(std::move(offline_in)),
      latency(std::move(latency_in)),
      download_throughput(std::move(download_throughput_in)),
      upload_throughput(std::move(upload_throughput_in)) {}

NetworkConditions::~NetworkConditions() = default;

bool NetworkConditions::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
ClearCacheUrlFilter::ClearCacheUrlFilter()
    : type(),
      domains(),
      origins() {}

ClearCacheUrlFilter::ClearCacheUrlFilter(
    ClearCacheUrlFilter::Type type_in,
    const std::vector<std::string>& domains_in,
    const std::vector<url::Origin>& origins_in)
    : type(std::move(type_in)),
      domains(std::move(domains_in)),
      origins(std::move(origins_in)) {}

ClearCacheUrlFilter::~ClearCacheUrlFilter() = default;

bool ClearCacheUrlFilter::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char NetworkContext::Name_[] = "network::mojom::NetworkContext";
bool NetworkContext::AddHSTSForTesting(const std::string& host, base::Time expiry, bool include_subdomains) {
  NOTREACHED();
  return false;
}

class NetworkContext_ClearNetworkingHistorySince_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_ClearNetworkingHistorySince_ForwardToCallback(
      NetworkContext::ClearNetworkingHistorySinceCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::ClearNetworkingHistorySinceCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearNetworkingHistorySince_ForwardToCallback);
};

class NetworkContext_ClearHttpCache_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_ClearHttpCache_ForwardToCallback(
      NetworkContext::ClearHttpCacheCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::ClearHttpCacheCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearHttpCache_ForwardToCallback);
};

class NetworkContext_CreateTCPServerSocket_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_CreateTCPServerSocket_ForwardToCallback(
      NetworkContext::CreateTCPServerSocketCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::CreateTCPServerSocketCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_CreateTCPServerSocket_ForwardToCallback);
};

class NetworkContext_CreateTCPConnectedSocket_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_CreateTCPConnectedSocket_ForwardToCallback(
      NetworkContext::CreateTCPConnectedSocketCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::CreateTCPConnectedSocketCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_CreateTCPConnectedSocket_ForwardToCallback);
};
class NetworkContext_AddHSTSForTesting_HandleSyncResponse
    : public mojo::MessageReceiver {
 public:
  NetworkContext_AddHSTSForTesting_HandleSyncResponse(
      bool* result)
      : result_(result) {
    DCHECK(!*result_);
  }
  bool Accept(mojo::Message* message) override;
 private:
  bool* result_;DISALLOW_COPY_AND_ASSIGN(NetworkContext_AddHSTSForTesting_HandleSyncResponse);
};

class NetworkContext_AddHSTSForTesting_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkContext_AddHSTSForTesting_ForwardToCallback(
      NetworkContext::AddHSTSForTestingCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkContext::AddHSTSForTestingCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkContext_AddHSTSForTesting_ForwardToCallback);
};

NetworkContextProxy::NetworkContextProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}
class NetworkContextProxy_CreateURLLoaderFactory_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_CreateURLLoaderFactory_Message(
      uint32_t message_flags
      , ::network::mojom::URLLoaderFactoryRequest param_url_loader_factory
      , uint32_t param_process_id
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CreateURLLoaderFactory_Name,
          message_flags)
      , param_url_loader_factory_(std::move(param_url_loader_factory))
      , param_process_id_(std::move(param_process_id)){}
  ~NetworkContextProxy_CreateURLLoaderFactory_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ::network::mojom::URLLoaderFactoryRequest param_url_loader_factory, uint32_t param_process_id) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_CreateURLLoaderFactory_Message>(
          kFlags
          , std::move(param_url_loader_factory)
          , std::move(param_process_id)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CreateURLLoaderFactory_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CreateURLLoaderFactory_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::URLLoaderFactoryRequestDataView>(
      param_url_loader_factory, &params->url_loader_factory, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->url_loader_factory),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid url_loader_factory in NetworkContext.CreateURLLoaderFactory request");
  params->process_id = param_process_id;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->CreateURLLoaderFactory(
        std::move(param_url_loader_factory_),
        std::move(param_process_id_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CreateURLLoaderFactory_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::URLLoaderFactoryRequestDataView>(
      param_url_loader_factory_, &params->url_loader_factory, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->url_loader_factory),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid url_loader_factory in NetworkContext.CreateURLLoaderFactory request");
  params->process_id = param_process_id_;
  }
  ::network::mojom::URLLoaderFactoryRequest param_url_loader_factory_;
  uint32_t param_process_id_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_CreateURLLoaderFactory_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_CreateURLLoaderFactory_Message::kMessageTag = {};

void NetworkContextProxy::CreateURLLoaderFactory(
    ::network::mojom::URLLoaderFactoryRequest in_url_loader_factory, uint32_t in_process_id) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::CreateURLLoaderFactory");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_CreateURLLoaderFactory_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_url_loader_factory), std::move(in_process_id));
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_GetCookieManager_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_GetCookieManager_Message(
      uint32_t message_flags
      , ::network::mojom::CookieManagerRequest param_cookie_manager
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_GetCookieManager_Name,
          message_flags)
      , param_cookie_manager_(std::move(param_cookie_manager)){}
  ~NetworkContextProxy_GetCookieManager_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ::network::mojom::CookieManagerRequest param_cookie_manager) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_GetCookieManager_Message>(
          kFlags
          , std::move(param_cookie_manager)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_GetCookieManager_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_GetCookieManager_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::CookieManagerRequestDataView>(
      param_cookie_manager, &params->cookie_manager, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->cookie_manager),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid cookie_manager in NetworkContext.GetCookieManager request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->GetCookieManager(
        std::move(param_cookie_manager_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_GetCookieManager_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::CookieManagerRequestDataView>(
      param_cookie_manager_, &params->cookie_manager, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->cookie_manager),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid cookie_manager in NetworkContext.GetCookieManager request");
  }
  ::network::mojom::CookieManagerRequest param_cookie_manager_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_GetCookieManager_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_GetCookieManager_Message::kMessageTag = {};

void NetworkContextProxy::GetCookieManager(
    ::network::mojom::CookieManagerRequest in_cookie_manager) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::GetCookieManager");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_GetCookieManager_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_cookie_manager));
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_GetRestrictedCookieManager_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_GetRestrictedCookieManager_Message(
      uint32_t message_flags
      , ::network::mojom::RestrictedCookieManagerRequest param_restricted_cookie_manager
      , int32_t param_render_process_id
      , int32_t param_render_frame_id
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_GetRestrictedCookieManager_Name,
          message_flags)
      , param_restricted_cookie_manager_(std::move(param_restricted_cookie_manager))
      , param_render_process_id_(std::move(param_render_process_id))
      , param_render_frame_id_(std::move(param_render_frame_id)){}
  ~NetworkContextProxy_GetRestrictedCookieManager_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ::network::mojom::RestrictedCookieManagerRequest param_restricted_cookie_manager, int32_t param_render_process_id, int32_t param_render_frame_id) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_GetRestrictedCookieManager_Message>(
          kFlags
          , std::move(param_restricted_cookie_manager)
          , std::move(param_render_process_id)
          , std::move(param_render_frame_id)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_GetRestrictedCookieManager_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_GetRestrictedCookieManager_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::RestrictedCookieManagerRequestDataView>(
      param_restricted_cookie_manager, &params->restricted_cookie_manager, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->restricted_cookie_manager),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid restricted_cookie_manager in NetworkContext.GetRestrictedCookieManager request");
  params->render_process_id = param_render_process_id;
  params->render_frame_id = param_render_frame_id;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->GetRestrictedCookieManager(
        std::move(param_restricted_cookie_manager_),
        std::move(param_render_process_id_),
        std::move(param_render_frame_id_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_GetRestrictedCookieManager_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::RestrictedCookieManagerRequestDataView>(
      param_restricted_cookie_manager_, &params->restricted_cookie_manager, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->restricted_cookie_manager),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid restricted_cookie_manager in NetworkContext.GetRestrictedCookieManager request");
  params->render_process_id = param_render_process_id_;
  params->render_frame_id = param_render_frame_id_;
  }
  ::network::mojom::RestrictedCookieManagerRequest param_restricted_cookie_manager_;
  int32_t param_render_process_id_;
  int32_t param_render_frame_id_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_GetRestrictedCookieManager_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_GetRestrictedCookieManager_Message::kMessageTag = {};

void NetworkContextProxy::GetRestrictedCookieManager(
    ::network::mojom::RestrictedCookieManagerRequest in_restricted_cookie_manager, int32_t in_render_process_id, int32_t in_render_frame_id) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::GetRestrictedCookieManager");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_GetRestrictedCookieManager_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_restricted_cookie_manager), std::move(in_render_process_id), std::move(in_render_frame_id));
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_ClearNetworkingHistorySince_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_ClearNetworkingHistorySince_Message(
      uint32_t message_flags
      , base::Time param_start_time
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearNetworkingHistorySince_Name,
          message_flags)
      , param_start_time_(std::move(param_start_time)){}
  ~NetworkContextProxy_ClearNetworkingHistorySince_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      base::Time param_start_time) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_ClearNetworkingHistorySince_Message>(
          kFlags
          , std::move(param_start_time)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearNetworkingHistorySince_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearNetworkingHistorySince_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->start_time)::BaseType::BufferWriter
      start_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_start_time, buffer, &start_time_writer, &serialization_context);
  params->start_time.Set(
      start_time_writer.is_null() ? nullptr : start_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->start_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null start_time in NetworkContext.ClearNetworkingHistorySince request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::ClearNetworkingHistorySinceCallback callback) {
    impl->ClearNetworkingHistorySince(
        std::move(param_start_time_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearNetworkingHistorySince_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->start_time)::BaseType::BufferWriter
      start_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_start_time_, buffer, &start_time_writer, serialization_context);
  params->start_time.Set(
      start_time_writer.is_null() ? nullptr : start_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->start_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null start_time in NetworkContext.ClearNetworkingHistorySince request");
  }
  base::Time param_start_time_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_ClearNetworkingHistorySince_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_ClearNetworkingHistorySince_Message::kMessageTag = {};

void NetworkContextProxy::ClearNetworkingHistorySince(
    base::Time in_start_time, ClearNetworkingHistorySinceCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::ClearNetworkingHistorySince");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_ClearNetworkingHistorySince_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_start_time));
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_ClearNetworkingHistorySince_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_ClearHttpCache_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_ClearHttpCache_Message(
      uint32_t message_flags
      , base::Time param_start_time
      , base::Time param_end_time
      , ClearCacheUrlFilterPtr param_filter
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearHttpCache_Name,
          message_flags)
      , param_start_time_(std::move(param_start_time))
      , param_end_time_(std::move(param_end_time))
      , param_filter_(std::move(param_filter)){}
  ~NetworkContextProxy_ClearHttpCache_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      base::Time param_start_time, base::Time param_end_time, ClearCacheUrlFilterPtr param_filter) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_ClearHttpCache_Message>(
          kFlags
          , std::move(param_start_time)
          , std::move(param_end_time)
          , std::move(param_filter)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearHttpCache_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearHttpCache_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->start_time)::BaseType::BufferWriter
      start_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_start_time, buffer, &start_time_writer, &serialization_context);
  params->start_time.Set(
      start_time_writer.is_null() ? nullptr : start_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->start_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null start_time in NetworkContext.ClearHttpCache request");
  typename decltype(params->end_time)::BaseType::BufferWriter
      end_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_end_time, buffer, &end_time_writer, &serialization_context);
  params->end_time.Set(
      end_time_writer.is_null() ? nullptr : end_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->end_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null end_time in NetworkContext.ClearHttpCache request");
  typename decltype(params->filter)::BaseType::BufferWriter
      filter_writer;
  mojo::internal::Serialize<::network::mojom::ClearCacheUrlFilterDataView>(
      param_filter, buffer, &filter_writer, &serialization_context);
  params->filter.Set(
      filter_writer.is_null() ? nullptr : filter_writer.data());
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::ClearHttpCacheCallback callback) {
    impl->ClearHttpCache(
        std::move(param_start_time_),
        std::move(param_end_time_),
        std::move(param_filter_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearHttpCache_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->start_time)::BaseType::BufferWriter
      start_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_start_time_, buffer, &start_time_writer, serialization_context);
  params->start_time.Set(
      start_time_writer.is_null() ? nullptr : start_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->start_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null start_time in NetworkContext.ClearHttpCache request");
  typename decltype(params->end_time)::BaseType::BufferWriter
      end_time_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_end_time_, buffer, &end_time_writer, serialization_context);
  params->end_time.Set(
      end_time_writer.is_null() ? nullptr : end_time_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->end_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null end_time in NetworkContext.ClearHttpCache request");
  typename decltype(params->filter)::BaseType::BufferWriter
      filter_writer;
  mojo::internal::Serialize<::network::mojom::ClearCacheUrlFilterDataView>(
      param_filter_, buffer, &filter_writer, serialization_context);
  params->filter.Set(
      filter_writer.is_null() ? nullptr : filter_writer.data());
  }
  base::Time param_start_time_;
  base::Time param_end_time_;
  ClearCacheUrlFilterPtr param_filter_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_ClearHttpCache_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_ClearHttpCache_Message::kMessageTag = {};

void NetworkContextProxy::ClearHttpCache(
    base::Time in_start_time, base::Time in_end_time, ClearCacheUrlFilterPtr in_filter, ClearHttpCacheCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::ClearHttpCache");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_ClearHttpCache_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_start_time), std::move(in_end_time), std::move(in_filter));
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_ClearHttpCache_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_SetNetworkConditions_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_SetNetworkConditions_Message(
      uint32_t message_flags
      , const std::string& param_profile_id
      , NetworkConditionsPtr param_conditions
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_SetNetworkConditions_Name,
          message_flags)
      , param_profile_id_(std::move(param_profile_id))
      , param_conditions_(std::move(param_conditions)){}
  ~NetworkContextProxy_SetNetworkConditions_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const std::string& param_profile_id, NetworkConditionsPtr param_conditions) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_SetNetworkConditions_Message>(
          kFlags
          , std::move(param_profile_id)
          , std::move(param_conditions)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_SetNetworkConditions_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_SetNetworkConditions_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->profile_id)::BaseType::BufferWriter
      profile_id_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_profile_id, buffer, &profile_id_writer, &serialization_context);
  params->profile_id.Set(
      profile_id_writer.is_null() ? nullptr : profile_id_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->profile_id.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null profile_id in NetworkContext.SetNetworkConditions request");
  typename decltype(params->conditions)::BaseType::BufferWriter
      conditions_writer;
  mojo::internal::Serialize<::network::mojom::NetworkConditionsDataView>(
      param_conditions, buffer, &conditions_writer, &serialization_context);
  params->conditions.Set(
      conditions_writer.is_null() ? nullptr : conditions_writer.data());
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->SetNetworkConditions(
        std::move(param_profile_id_),
        std::move(param_conditions_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_SetNetworkConditions_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->profile_id)::BaseType::BufferWriter
      profile_id_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_profile_id_, buffer, &profile_id_writer, serialization_context);
  params->profile_id.Set(
      profile_id_writer.is_null() ? nullptr : profile_id_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->profile_id.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null profile_id in NetworkContext.SetNetworkConditions request");
  typename decltype(params->conditions)::BaseType::BufferWriter
      conditions_writer;
  mojo::internal::Serialize<::network::mojom::NetworkConditionsDataView>(
      param_conditions_, buffer, &conditions_writer, serialization_context);
  params->conditions.Set(
      conditions_writer.is_null() ? nullptr : conditions_writer.data());
  }
  std::string param_profile_id_;
  NetworkConditionsPtr param_conditions_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_SetNetworkConditions_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_SetNetworkConditions_Message::kMessageTag = {};

void NetworkContextProxy::SetNetworkConditions(
    const std::string& in_profile_id, NetworkConditionsPtr in_conditions) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::SetNetworkConditions");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_SetNetworkConditions_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_profile_id), std::move(in_conditions));
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_SetAcceptLanguage_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_SetAcceptLanguage_Message(
      uint32_t message_flags
      , const std::string& param_new_accept_language
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_SetAcceptLanguage_Name,
          message_flags)
      , param_new_accept_language_(std::move(param_new_accept_language)){}
  ~NetworkContextProxy_SetAcceptLanguage_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const std::string& param_new_accept_language) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_SetAcceptLanguage_Message>(
          kFlags
          , std::move(param_new_accept_language)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_SetAcceptLanguage_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_SetAcceptLanguage_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->new_accept_language)::BaseType::BufferWriter
      new_accept_language_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_new_accept_language, buffer, &new_accept_language_writer, &serialization_context);
  params->new_accept_language.Set(
      new_accept_language_writer.is_null() ? nullptr : new_accept_language_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->new_accept_language.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null new_accept_language in NetworkContext.SetAcceptLanguage request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->SetAcceptLanguage(
        std::move(param_new_accept_language_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_SetAcceptLanguage_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->new_accept_language)::BaseType::BufferWriter
      new_accept_language_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_new_accept_language_, buffer, &new_accept_language_writer, serialization_context);
  params->new_accept_language.Set(
      new_accept_language_writer.is_null() ? nullptr : new_accept_language_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->new_accept_language.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null new_accept_language in NetworkContext.SetAcceptLanguage request");
  }
  std::string param_new_accept_language_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_SetAcceptLanguage_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_SetAcceptLanguage_Message::kMessageTag = {};

void NetworkContextProxy::SetAcceptLanguage(
    const std::string& in_new_accept_language) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::SetAcceptLanguage");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_SetAcceptLanguage_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_new_accept_language));
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_SetCTPolicy_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_SetCTPolicy_Message(
      uint32_t message_flags
      , const std::vector<std::string>& param_required_hosts
      , const std::vector<std::string>& param_excluded_hosts
      , const std::vector<std::string>& param_excluded_spkis
      , const std::vector<std::string>& param_excluded_legacy_spkis
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_SetCTPolicy_Name,
          message_flags)
      , param_required_hosts_(std::move(param_required_hosts))
      , param_excluded_hosts_(std::move(param_excluded_hosts))
      , param_excluded_spkis_(std::move(param_excluded_spkis))
      , param_excluded_legacy_spkis_(std::move(param_excluded_legacy_spkis)){}
  ~NetworkContextProxy_SetCTPolicy_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const std::vector<std::string>& param_required_hosts, const std::vector<std::string>& param_excluded_hosts, const std::vector<std::string>& param_excluded_spkis, const std::vector<std::string>& param_excluded_legacy_spkis) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_SetCTPolicy_Message>(
          kFlags
          , std::move(param_required_hosts)
          , std::move(param_excluded_hosts)
          , std::move(param_excluded_spkis)
          , std::move(param_excluded_legacy_spkis)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_SetCTPolicy_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_SetCTPolicy_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->required_hosts)::BaseType::BufferWriter
      required_hosts_writer;
  const mojo::internal::ContainerValidateParams required_hosts_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
      param_required_hosts, buffer, &required_hosts_writer, &required_hosts_validate_params,
      &serialization_context);
  params->required_hosts.Set(
      required_hosts_writer.is_null() ? nullptr : required_hosts_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->required_hosts.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null required_hosts in NetworkContext.SetCTPolicy request");
  typename decltype(params->excluded_hosts)::BaseType::BufferWriter
      excluded_hosts_writer;
  const mojo::internal::ContainerValidateParams excluded_hosts_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
      param_excluded_hosts, buffer, &excluded_hosts_writer, &excluded_hosts_validate_params,
      &serialization_context);
  params->excluded_hosts.Set(
      excluded_hosts_writer.is_null() ? nullptr : excluded_hosts_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->excluded_hosts.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null excluded_hosts in NetworkContext.SetCTPolicy request");
  typename decltype(params->excluded_spkis)::BaseType::BufferWriter
      excluded_spkis_writer;
  const mojo::internal::ContainerValidateParams excluded_spkis_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
      param_excluded_spkis, buffer, &excluded_spkis_writer, &excluded_spkis_validate_params,
      &serialization_context);
  params->excluded_spkis.Set(
      excluded_spkis_writer.is_null() ? nullptr : excluded_spkis_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->excluded_spkis.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null excluded_spkis in NetworkContext.SetCTPolicy request");
  typename decltype(params->excluded_legacy_spkis)::BaseType::BufferWriter
      excluded_legacy_spkis_writer;
  const mojo::internal::ContainerValidateParams excluded_legacy_spkis_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
      param_excluded_legacy_spkis, buffer, &excluded_legacy_spkis_writer, &excluded_legacy_spkis_validate_params,
      &serialization_context);
  params->excluded_legacy_spkis.Set(
      excluded_legacy_spkis_writer.is_null() ? nullptr : excluded_legacy_spkis_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->excluded_legacy_spkis.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null excluded_legacy_spkis in NetworkContext.SetCTPolicy request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->SetCTPolicy(
        std::move(param_required_hosts_),
        std::move(param_excluded_hosts_),
        std::move(param_excluded_spkis_),
        std::move(param_excluded_legacy_spkis_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_SetCTPolicy_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->required_hosts)::BaseType::BufferWriter
      required_hosts_writer;
  const mojo::internal::ContainerValidateParams required_hosts_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
      param_required_hosts_, buffer, &required_hosts_writer, &required_hosts_validate_params,
      serialization_context);
  params->required_hosts.Set(
      required_hosts_writer.is_null() ? nullptr : required_hosts_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->required_hosts.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null required_hosts in NetworkContext.SetCTPolicy request");
  typename decltype(params->excluded_hosts)::BaseType::BufferWriter
      excluded_hosts_writer;
  const mojo::internal::ContainerValidateParams excluded_hosts_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
      param_excluded_hosts_, buffer, &excluded_hosts_writer, &excluded_hosts_validate_params,
      serialization_context);
  params->excluded_hosts.Set(
      excluded_hosts_writer.is_null() ? nullptr : excluded_hosts_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->excluded_hosts.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null excluded_hosts in NetworkContext.SetCTPolicy request");
  typename decltype(params->excluded_spkis)::BaseType::BufferWriter
      excluded_spkis_writer;
  const mojo::internal::ContainerValidateParams excluded_spkis_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
      param_excluded_spkis_, buffer, &excluded_spkis_writer, &excluded_spkis_validate_params,
      serialization_context);
  params->excluded_spkis.Set(
      excluded_spkis_writer.is_null() ? nullptr : excluded_spkis_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->excluded_spkis.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null excluded_spkis in NetworkContext.SetCTPolicy request");
  typename decltype(params->excluded_legacy_spkis)::BaseType::BufferWriter
      excluded_legacy_spkis_writer;
  const mojo::internal::ContainerValidateParams excluded_legacy_spkis_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
      param_excluded_legacy_spkis_, buffer, &excluded_legacy_spkis_writer, &excluded_legacy_spkis_validate_params,
      serialization_context);
  params->excluded_legacy_spkis.Set(
      excluded_legacy_spkis_writer.is_null() ? nullptr : excluded_legacy_spkis_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->excluded_legacy_spkis.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null excluded_legacy_spkis in NetworkContext.SetCTPolicy request");
  }
  std::vector<std::string> param_required_hosts_;
  std::vector<std::string> param_excluded_hosts_;
  std::vector<std::string> param_excluded_spkis_;
  std::vector<std::string> param_excluded_legacy_spkis_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_SetCTPolicy_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_SetCTPolicy_Message::kMessageTag = {};

void NetworkContextProxy::SetCTPolicy(
    const std::vector<std::string>& in_required_hosts, const std::vector<std::string>& in_excluded_hosts, const std::vector<std::string>& in_excluded_spkis, const std::vector<std::string>& in_excluded_legacy_spkis) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::SetCTPolicy");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_SetCTPolicy_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_required_hosts), std::move(in_excluded_hosts), std::move(in_excluded_spkis), std::move(in_excluded_legacy_spkis));
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_CreateUDPSocket_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_CreateUDPSocket_Message(
      uint32_t message_flags
      , ::network::mojom::UDPSocketRequest param_request
      , ::network::mojom::UDPSocketReceiverPtr param_receiver
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CreateUDPSocket_Name,
          message_flags)
      , param_request_(std::move(param_request))
      , param_receiver_(param_receiver.PassInterface()){}
  ~NetworkContextProxy_CreateUDPSocket_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ::network::mojom::UDPSocketRequest param_request, ::network::mojom::UDPSocketReceiverPtr param_receiver) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_CreateUDPSocket_Message>(
          kFlags
          , std::move(param_request)
          , std::move(param_receiver)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CreateUDPSocket_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CreateUDPSocket_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::UDPSocketRequestDataView>(
      param_request, &params->request, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->request),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid request in NetworkContext.CreateUDPSocket request");
  mojo::internal::Serialize<::network::mojom::UDPSocketReceiverPtrDataView>(
      param_receiver, &params->receiver, &serialization_context);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->CreateUDPSocket(
        std::move(param_request_),
        ::network::mojom::UDPSocketReceiverPtr(std::move(param_receiver_)));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CreateUDPSocket_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::UDPSocketRequestDataView>(
      param_request_, &params->request, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->request),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid request in NetworkContext.CreateUDPSocket request");
  mojo::internal::Serialize<::network::mojom::UDPSocketReceiverPtrDataView>(
      param_receiver_, &params->receiver, serialization_context);
  }
  ::network::mojom::UDPSocketRequest param_request_;
  ::network::mojom::UDPSocketReceiverPtrInfo param_receiver_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_CreateUDPSocket_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_CreateUDPSocket_Message::kMessageTag = {};

void NetworkContextProxy::CreateUDPSocket(
    ::network::mojom::UDPSocketRequest in_request, ::network::mojom::UDPSocketReceiverPtr in_receiver) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::CreateUDPSocket");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_CreateUDPSocket_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_request), std::move(in_receiver));
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_CreateTCPServerSocket_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_CreateTCPServerSocket_Message(
      uint32_t message_flags
      , const net::IPEndPoint& param_local_addr
      , uint32_t param_backlog
      , const net::MutableNetworkTrafficAnnotationTag& param_traffic_annotation
      , ::network::mojom::TCPServerSocketRequest param_socket
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CreateTCPServerSocket_Name,
          message_flags)
      , param_local_addr_(std::move(param_local_addr))
      , param_backlog_(std::move(param_backlog))
      , param_traffic_annotation_(std::move(param_traffic_annotation))
      , param_socket_(std::move(param_socket)){}
  ~NetworkContextProxy_CreateTCPServerSocket_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const net::IPEndPoint& param_local_addr, uint32_t param_backlog, const net::MutableNetworkTrafficAnnotationTag& param_traffic_annotation, ::network::mojom::TCPServerSocketRequest param_socket) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_CreateTCPServerSocket_Message>(
          kFlags
          , std::move(param_local_addr)
          , std::move(param_backlog)
          , std::move(param_traffic_annotation)
          , std::move(param_socket)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CreateTCPServerSocket_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CreateTCPServerSocket_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->local_addr)::BaseType::BufferWriter
      local_addr_writer;
  mojo::internal::Serialize<::net::interfaces::IPEndPointDataView>(
      param_local_addr, buffer, &local_addr_writer, &serialization_context);
  params->local_addr.Set(
      local_addr_writer.is_null() ? nullptr : local_addr_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->local_addr.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null local_addr in NetworkContext.CreateTCPServerSocket request");
  params->backlog = param_backlog;
  typename decltype(params->traffic_annotation)::BaseType::BufferWriter
      traffic_annotation_writer;
  mojo::internal::Serialize<::network::mojom::MutableNetworkTrafficAnnotationTagDataView>(
      param_traffic_annotation, buffer, &traffic_annotation_writer, &serialization_context);
  params->traffic_annotation.Set(
      traffic_annotation_writer.is_null() ? nullptr : traffic_annotation_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->traffic_annotation.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null traffic_annotation in NetworkContext.CreateTCPServerSocket request");
  mojo::internal::Serialize<::network::mojom::TCPServerSocketRequestDataView>(
      param_socket, &params->socket, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->socket),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid socket in NetworkContext.CreateTCPServerSocket request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::CreateTCPServerSocketCallback callback) {
    impl->CreateTCPServerSocket(
        std::move(param_local_addr_),
        std::move(param_backlog_),
        std::move(param_traffic_annotation_),
        std::move(param_socket_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CreateTCPServerSocket_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->local_addr)::BaseType::BufferWriter
      local_addr_writer;
  mojo::internal::Serialize<::net::interfaces::IPEndPointDataView>(
      param_local_addr_, buffer, &local_addr_writer, serialization_context);
  params->local_addr.Set(
      local_addr_writer.is_null() ? nullptr : local_addr_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->local_addr.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null local_addr in NetworkContext.CreateTCPServerSocket request");
  params->backlog = param_backlog_;
  typename decltype(params->traffic_annotation)::BaseType::BufferWriter
      traffic_annotation_writer;
  mojo::internal::Serialize<::network::mojom::MutableNetworkTrafficAnnotationTagDataView>(
      param_traffic_annotation_, buffer, &traffic_annotation_writer, serialization_context);
  params->traffic_annotation.Set(
      traffic_annotation_writer.is_null() ? nullptr : traffic_annotation_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->traffic_annotation.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null traffic_annotation in NetworkContext.CreateTCPServerSocket request");
  mojo::internal::Serialize<::network::mojom::TCPServerSocketRequestDataView>(
      param_socket_, &params->socket, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->socket),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid socket in NetworkContext.CreateTCPServerSocket request");
  }
  net::IPEndPoint param_local_addr_;
  uint32_t param_backlog_;
  net::MutableNetworkTrafficAnnotationTag param_traffic_annotation_;
  ::network::mojom::TCPServerSocketRequest param_socket_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_CreateTCPServerSocket_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_CreateTCPServerSocket_Message::kMessageTag = {};

void NetworkContextProxy::CreateTCPServerSocket(
    const net::IPEndPoint& in_local_addr, uint32_t in_backlog, const net::MutableNetworkTrafficAnnotationTag& in_traffic_annotation, ::network::mojom::TCPServerSocketRequest in_socket, CreateTCPServerSocketCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::CreateTCPServerSocket");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_CreateTCPServerSocket_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_local_addr), std::move(in_backlog), std::move(in_traffic_annotation), std::move(in_socket));
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_CreateTCPServerSocket_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_CreateTCPConnectedSocket_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_CreateTCPConnectedSocket_Message(
      uint32_t message_flags
      , const base::Optional<net::IPEndPoint>& param_local_addr
      , const net::AddressList& param_remote_addr_list
      , const net::MutableNetworkTrafficAnnotationTag& param_traffic_annotation
      , ::network::mojom::TCPConnectedSocketRequest param_socket
      , ::network::mojom::TCPConnectedSocketObserverPtr param_observer
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CreateTCPConnectedSocket_Name,
          message_flags)
      , param_local_addr_(std::move(param_local_addr))
      , param_remote_addr_list_(std::move(param_remote_addr_list))
      , param_traffic_annotation_(std::move(param_traffic_annotation))
      , param_socket_(std::move(param_socket))
      , param_observer_(param_observer.PassInterface()){}
  ~NetworkContextProxy_CreateTCPConnectedSocket_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const base::Optional<net::IPEndPoint>& param_local_addr, const net::AddressList& param_remote_addr_list, const net::MutableNetworkTrafficAnnotationTag& param_traffic_annotation, ::network::mojom::TCPConnectedSocketRequest param_socket, ::network::mojom::TCPConnectedSocketObserverPtr param_observer) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_CreateTCPConnectedSocket_Message>(
          kFlags
          , std::move(param_local_addr)
          , std::move(param_remote_addr_list)
          , std::move(param_traffic_annotation)
          , std::move(param_socket)
          , std::move(param_observer)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CreateTCPConnectedSocket_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CreateTCPConnectedSocket_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->local_addr)::BaseType::BufferWriter
      local_addr_writer;
  mojo::internal::Serialize<::net::interfaces::IPEndPointDataView>(
      param_local_addr, buffer, &local_addr_writer, &serialization_context);
  params->local_addr.Set(
      local_addr_writer.is_null() ? nullptr : local_addr_writer.data());
  typename decltype(params->remote_addr_list)::BaseType::BufferWriter
      remote_addr_list_writer;
  mojo::internal::Serialize<::net::interfaces::AddressListDataView>(
      param_remote_addr_list, buffer, &remote_addr_list_writer, &serialization_context);
  params->remote_addr_list.Set(
      remote_addr_list_writer.is_null() ? nullptr : remote_addr_list_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->remote_addr_list.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null remote_addr_list in NetworkContext.CreateTCPConnectedSocket request");
  typename decltype(params->traffic_annotation)::BaseType::BufferWriter
      traffic_annotation_writer;
  mojo::internal::Serialize<::network::mojom::MutableNetworkTrafficAnnotationTagDataView>(
      param_traffic_annotation, buffer, &traffic_annotation_writer, &serialization_context);
  params->traffic_annotation.Set(
      traffic_annotation_writer.is_null() ? nullptr : traffic_annotation_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->traffic_annotation.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null traffic_annotation in NetworkContext.CreateTCPConnectedSocket request");
  mojo::internal::Serialize<::network::mojom::TCPConnectedSocketRequestDataView>(
      param_socket, &params->socket, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->socket),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid socket in NetworkContext.CreateTCPConnectedSocket request");
  mojo::internal::Serialize<::network::mojom::TCPConnectedSocketObserverPtrDataView>(
      param_observer, &params->observer, &serialization_context);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::CreateTCPConnectedSocketCallback callback) {
    impl->CreateTCPConnectedSocket(
        std::move(param_local_addr_),
        std::move(param_remote_addr_list_),
        std::move(param_traffic_annotation_),
        std::move(param_socket_),
        ::network::mojom::TCPConnectedSocketObserverPtr(std::move(param_observer_)), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CreateTCPConnectedSocket_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->local_addr)::BaseType::BufferWriter
      local_addr_writer;
  mojo::internal::Serialize<::net::interfaces::IPEndPointDataView>(
      param_local_addr_, buffer, &local_addr_writer, serialization_context);
  params->local_addr.Set(
      local_addr_writer.is_null() ? nullptr : local_addr_writer.data());
  typename decltype(params->remote_addr_list)::BaseType::BufferWriter
      remote_addr_list_writer;
  mojo::internal::Serialize<::net::interfaces::AddressListDataView>(
      param_remote_addr_list_, buffer, &remote_addr_list_writer, serialization_context);
  params->remote_addr_list.Set(
      remote_addr_list_writer.is_null() ? nullptr : remote_addr_list_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->remote_addr_list.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null remote_addr_list in NetworkContext.CreateTCPConnectedSocket request");
  typename decltype(params->traffic_annotation)::BaseType::BufferWriter
      traffic_annotation_writer;
  mojo::internal::Serialize<::network::mojom::MutableNetworkTrafficAnnotationTagDataView>(
      param_traffic_annotation_, buffer, &traffic_annotation_writer, serialization_context);
  params->traffic_annotation.Set(
      traffic_annotation_writer.is_null() ? nullptr : traffic_annotation_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->traffic_annotation.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null traffic_annotation in NetworkContext.CreateTCPConnectedSocket request");
  mojo::internal::Serialize<::network::mojom::TCPConnectedSocketRequestDataView>(
      param_socket_, &params->socket, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->socket),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid socket in NetworkContext.CreateTCPConnectedSocket request");
  mojo::internal::Serialize<::network::mojom::TCPConnectedSocketObserverPtrDataView>(
      param_observer_, &params->observer, serialization_context);
  }
  base::Optional<net::IPEndPoint> param_local_addr_;
  net::AddressList param_remote_addr_list_;
  net::MutableNetworkTrafficAnnotationTag param_traffic_annotation_;
  ::network::mojom::TCPConnectedSocketRequest param_socket_;
  ::network::mojom::TCPConnectedSocketObserverPtrInfo param_observer_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_CreateTCPConnectedSocket_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_CreateTCPConnectedSocket_Message::kMessageTag = {};

void NetworkContextProxy::CreateTCPConnectedSocket(
    const base::Optional<net::IPEndPoint>& in_local_addr, const net::AddressList& in_remote_addr_list, const net::MutableNetworkTrafficAnnotationTag& in_traffic_annotation, ::network::mojom::TCPConnectedSocketRequest in_socket, ::network::mojom::TCPConnectedSocketObserverPtr in_observer, CreateTCPConnectedSocketCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::CreateTCPConnectedSocket");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_CreateTCPConnectedSocket_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_local_addr), std::move(in_remote_addr_list), std::move(in_traffic_annotation), std::move(in_socket), std::move(in_observer));
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_CreateTCPConnectedSocket_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContextProxy_CreateWebSocket_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_CreateWebSocket_Message(
      uint32_t message_flags
      , ::network::mojom::WebSocketRequest param_request
      , int32_t param_process_id
      , int32_t param_render_frame_id
      , const url::Origin& param_origin
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CreateWebSocket_Name,
          message_flags)
      , param_request_(std::move(param_request))
      , param_process_id_(std::move(param_process_id))
      , param_render_frame_id_(std::move(param_render_frame_id))
      , param_origin_(std::move(param_origin)){}
  ~NetworkContextProxy_CreateWebSocket_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ::network::mojom::WebSocketRequest param_request, int32_t param_process_id, int32_t param_render_frame_id, const url::Origin& param_origin) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_CreateWebSocket_Message>(
          kFlags
          , std::move(param_request)
          , std::move(param_process_id)
          , std::move(param_render_frame_id)
          , std::move(param_origin)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CreateWebSocket_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CreateWebSocket_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::WebSocketRequestDataView>(
      param_request, &params->request, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->request),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid request in NetworkContext.CreateWebSocket request");
  params->process_id = param_process_id;
  params->render_frame_id = param_render_frame_id;
  typename decltype(params->origin)::BaseType::BufferWriter
      origin_writer;
  mojo::internal::Serialize<::url::mojom::OriginDataView>(
      param_origin, buffer, &origin_writer, &serialization_context);
  params->origin.Set(
      origin_writer.is_null() ? nullptr : origin_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->origin.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null origin in NetworkContext.CreateWebSocket request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl) {
    impl->CreateWebSocket(
        std::move(param_request_),
        std::move(param_process_id_),
        std::move(param_render_frame_id_),
        std::move(param_origin_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CreateWebSocket_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::WebSocketRequestDataView>(
      param_request_, &params->request, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->request),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid request in NetworkContext.CreateWebSocket request");
  params->process_id = param_process_id_;
  params->render_frame_id = param_render_frame_id_;
  typename decltype(params->origin)::BaseType::BufferWriter
      origin_writer;
  mojo::internal::Serialize<::url::mojom::OriginDataView>(
      param_origin_, buffer, &origin_writer, serialization_context);
  params->origin.Set(
      origin_writer.is_null() ? nullptr : origin_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->origin.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null origin in NetworkContext.CreateWebSocket request");
  }
  ::network::mojom::WebSocketRequest param_request_;
  int32_t param_process_id_;
  int32_t param_render_frame_id_;
  url::Origin param_origin_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_CreateWebSocket_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_CreateWebSocket_Message::kMessageTag = {};

void NetworkContextProxy::CreateWebSocket(
    ::network::mojom::WebSocketRequest in_request, int32_t in_process_id, int32_t in_render_frame_id, const url::Origin& in_origin) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::CreateWebSocket");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_CreateWebSocket_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_request), std::move(in_process_id), std::move(in_render_frame_id), std::move(in_origin));
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkContextProxy_AddHSTSForTesting_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContextProxy_AddHSTSForTesting_Message(
      uint32_t message_flags
      , const std::string& param_host
      , base::Time param_expiry
      , bool param_include_subdomains
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_AddHSTSForTesting_Name,
          message_flags)
      , param_host_(std::move(param_host))
      , param_expiry_(std::move(param_expiry))
      , param_include_subdomains_(std::move(param_include_subdomains)){}
  ~NetworkContextProxy_AddHSTSForTesting_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      const std::string& param_host, base::Time param_expiry, bool param_include_subdomains) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContextProxy_AddHSTSForTesting_Message>(
          kFlags
          , std::move(param_host)
          , std::move(param_expiry)
          , std::move(param_include_subdomains)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_AddHSTSForTesting_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_AddHSTSForTesting_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->host)::BaseType::BufferWriter
      host_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_host, buffer, &host_writer, &serialization_context);
  params->host.Set(
      host_writer.is_null() ? nullptr : host_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->host.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null host in NetworkContext.AddHSTSForTesting request");
  typename decltype(params->expiry)::BaseType::BufferWriter
      expiry_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_expiry, buffer, &expiry_writer, &serialization_context);
  params->expiry.Set(
      expiry_writer.is_null() ? nullptr : expiry_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->expiry.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null expiry in NetworkContext.AddHSTSForTesting request");
  params->include_subdomains = param_include_subdomains;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext* impl, NetworkContext::AddHSTSForTestingCallback callback) {
    impl->AddHSTSForTesting(
        std::move(param_host_),
        std::move(param_expiry_),
        std::move(param_include_subdomains_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_AddHSTSForTesting_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->host)::BaseType::BufferWriter
      host_writer;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_host_, buffer, &host_writer, serialization_context);
  params->host.Set(
      host_writer.is_null() ? nullptr : host_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->host.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null host in NetworkContext.AddHSTSForTesting request");
  typename decltype(params->expiry)::BaseType::BufferWriter
      expiry_writer;
  mojo::internal::Serialize<::mojo_base::mojom::TimeDataView>(
      param_expiry_, buffer, &expiry_writer, serialization_context);
  params->expiry.Set(
      expiry_writer.is_null() ? nullptr : expiry_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->expiry.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null expiry in NetworkContext.AddHSTSForTesting request");
  params->include_subdomains = param_include_subdomains_;
  }
  std::string param_host_;
  base::Time param_expiry_;
  bool param_include_subdomains_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContextProxy_AddHSTSForTesting_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContextProxy_AddHSTSForTesting_Message::kMessageTag = {};
bool NetworkContextProxy::AddHSTSForTesting(
    const std::string& param_host, base::Time param_expiry, bool param_include_subdomains) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::AddHSTSForTesting");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = true;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_AddHSTSForTesting_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(param_host), std::move(param_expiry), std::move(param_include_subdomains));

  bool result = false;
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_AddHSTSForTesting_HandleSyncResponse(
          &result));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
  return result;
}

void NetworkContextProxy::AddHSTSForTesting(
    const std::string& in_host, base::Time in_expiry, bool in_include_subdomains, AddHSTSForTestingCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkContext::AddHSTSForTesting");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkContextProxy_AddHSTSForTesting_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_host), std::move(in_expiry), std::move(in_include_subdomains));
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkContext_AddHSTSForTesting_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkContext_ClearNetworkingHistorySince_ProxyToResponder {
 public:
  static NetworkContext::ClearNetworkingHistorySinceCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_ClearNetworkingHistorySince_ProxyToResponder> proxy(
        new NetworkContext_ClearNetworkingHistorySince_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_ClearNetworkingHistorySince_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_ClearNetworkingHistorySince_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_ClearNetworkingHistorySince_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::ClearNetworkingHistorySinceCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearNetworkingHistorySince_ProxyToResponder);
};
class NetworkContext_ClearNetworkingHistorySince_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_ClearNetworkingHistorySince_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearNetworkingHistorySince_Name,
          message_flags){}
  ~NetworkContext_ClearNetworkingHistorySince_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_ClearNetworkingHistorySince_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearNetworkingHistorySince_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::ClearNetworkingHistorySinceCallback* callback) {
    std::move(*callback).Run();
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearNetworkingHistorySince_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_ClearNetworkingHistorySince_Response_Message::kMessageTag = {};

bool NetworkContext_ClearNetworkingHistorySince_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::ClearNetworkingHistorySinceCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_ClearNetworkingHistorySince_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_ClearNetworkingHistorySince_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::ClearNetworkingHistorySince response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_ClearNetworkingHistorySince_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_ClearNetworkingHistorySince_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearNetworkingHistorySinceCallback",
               "message", message.name());
#endif
  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_ClearHttpCache_ProxyToResponder {
 public:
  static NetworkContext::ClearHttpCacheCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_ClearHttpCache_ProxyToResponder> proxy(
        new NetworkContext_ClearHttpCache_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_ClearHttpCache_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_ClearHttpCache_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_ClearHttpCache_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::ClearHttpCacheCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearHttpCache_ProxyToResponder);
};
class NetworkContext_ClearHttpCache_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_ClearHttpCache_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_ClearHttpCache_Name,
          message_flags){}
  ~NetworkContext_ClearHttpCache_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_ClearHttpCache_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_ClearHttpCache_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_ClearHttpCache_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::ClearHttpCacheCallback* callback) {
    std::move(*callback).Run();
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_ClearHttpCache_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_ClearHttpCache_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_ClearHttpCache_Response_Message::kMessageTag = {};

bool NetworkContext_ClearHttpCache_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::ClearHttpCacheCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_ClearHttpCache_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_ClearHttpCache_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_ClearHttpCache_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_ClearHttpCache_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::ClearHttpCache response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_ClearHttpCache_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_ClearHttpCache_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearHttpCacheCallback",
               "message", message.name());
#endif
  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_CreateTCPServerSocket_ProxyToResponder {
 public:
  static NetworkContext::CreateTCPServerSocketCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_CreateTCPServerSocket_ProxyToResponder> proxy(
        new NetworkContext_CreateTCPServerSocket_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_CreateTCPServerSocket_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_CreateTCPServerSocket_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_CreateTCPServerSocket_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::CreateTCPServerSocketCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      int32_t in_result, const base::Optional<net::IPEndPoint>& in_local_addr_out);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_CreateTCPServerSocket_ProxyToResponder);
};
class NetworkContext_CreateTCPServerSocket_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_CreateTCPServerSocket_Response_Message(
      uint32_t message_flags
      , int32_t param_result
      , const base::Optional<net::IPEndPoint>& param_local_addr_out
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CreateTCPServerSocket_Name,
          message_flags)
      , param_result_(std::move(param_result))
      , param_local_addr_out_(std::move(param_local_addr_out)){}
  ~NetworkContext_CreateTCPServerSocket_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      int32_t param_result, const base::Optional<net::IPEndPoint>& param_local_addr_out) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_CreateTCPServerSocket_Response_Message>(
          kFlags
          , std::move(param_result)
          , std::move(param_local_addr_out)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CreateTCPServerSocket_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CreateTCPServerSocket_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->result = param_result;
  typename decltype(params->local_addr_out)::BaseType::BufferWriter
      local_addr_out_writer;
  mojo::internal::Serialize<::net::interfaces::IPEndPointDataView>(
      param_local_addr_out, buffer, &local_addr_out_writer, &serialization_context);
  params->local_addr_out.Set(
      local_addr_out_writer.is_null() ? nullptr : local_addr_out_writer.data());
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::CreateTCPServerSocketCallback* callback) {
    std::move(*callback).Run(
        std::move(param_result_), 
        std::move(param_local_addr_out_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CreateTCPServerSocket_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->result = param_result_;
  typename decltype(params->local_addr_out)::BaseType::BufferWriter
      local_addr_out_writer;
  mojo::internal::Serialize<::net::interfaces::IPEndPointDataView>(
      param_local_addr_out_, buffer, &local_addr_out_writer, serialization_context);
  params->local_addr_out.Set(
      local_addr_out_writer.is_null() ? nullptr : local_addr_out_writer.data());
  }
  int32_t param_result_;
  base::Optional<net::IPEndPoint> param_local_addr_out_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_CreateTCPServerSocket_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_CreateTCPServerSocket_Response_Message::kMessageTag = {};

bool NetworkContext_CreateTCPServerSocket_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::CreateTCPServerSocketCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_CreateTCPServerSocket_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_CreateTCPServerSocket_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_CreateTCPServerSocket_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  int32_t p_result{};
  base::Optional<net::IPEndPoint> p_local_addr_out{};
  NetworkContext_CreateTCPServerSocket_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_result = input_data_view.result();
  if (!input_data_view.ReadLocalAddrOut(&p_local_addr_out))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::CreateTCPServerSocket response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_result), 
std::move(p_local_addr_out));
  return true;
}

void NetworkContext_CreateTCPServerSocket_ProxyToResponder::Run(
    int32_t in_result, const base::Optional<net::IPEndPoint>& in_local_addr_out) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_CreateTCPServerSocket_Response_Message::Build(kSerialize, is_sync_, std::move(in_result), std::move(in_local_addr_out));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateTCPServerSocketCallback",
               "message", message.name());
#endif
  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_CreateTCPConnectedSocket_ProxyToResponder {
 public:
  static NetworkContext::CreateTCPConnectedSocketCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_CreateTCPConnectedSocket_ProxyToResponder> proxy(
        new NetworkContext_CreateTCPConnectedSocket_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_CreateTCPConnectedSocket_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_CreateTCPConnectedSocket_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_CreateTCPConnectedSocket_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::CreateTCPConnectedSocketCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      int32_t in_result, mojo::ScopedDataPipeConsumerHandle in_receive_stream, mojo::ScopedDataPipeProducerHandle in_send_stream);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_CreateTCPConnectedSocket_ProxyToResponder);
};
class NetworkContext_CreateTCPConnectedSocket_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_CreateTCPConnectedSocket_Response_Message(
      uint32_t message_flags
      , int32_t param_result
      , mojo::ScopedDataPipeConsumerHandle param_receive_stream
      , mojo::ScopedDataPipeProducerHandle param_send_stream
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_CreateTCPConnectedSocket_Name,
          message_flags)
      , param_result_(std::move(param_result))
      , param_receive_stream_(std::move(param_receive_stream))
      , param_send_stream_(std::move(param_send_stream)){}
  ~NetworkContext_CreateTCPConnectedSocket_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      int32_t param_result, mojo::ScopedDataPipeConsumerHandle param_receive_stream, mojo::ScopedDataPipeProducerHandle param_send_stream) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_CreateTCPConnectedSocket_Response_Message>(
          kFlags
          , std::move(param_result)
          , std::move(param_receive_stream)
          , std::move(param_send_stream)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_CreateTCPConnectedSocket_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_CreateTCPConnectedSocket_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->result = param_result;
  mojo::internal::Serialize<mojo::ScopedDataPipeConsumerHandle>(
      param_receive_stream, &params->receive_stream, &serialization_context);
  mojo::internal::Serialize<mojo::ScopedDataPipeProducerHandle>(
      param_send_stream, &params->send_stream, &serialization_context);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::CreateTCPConnectedSocketCallback* callback) {
    std::move(*callback).Run(
        std::move(param_result_), 
        std::move(param_receive_stream_), 
        std::move(param_send_stream_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_CreateTCPConnectedSocket_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->result = param_result_;
  mojo::internal::Serialize<mojo::ScopedDataPipeConsumerHandle>(
      param_receive_stream_, &params->receive_stream, serialization_context);
  mojo::internal::Serialize<mojo::ScopedDataPipeProducerHandle>(
      param_send_stream_, &params->send_stream, serialization_context);
  }
  int32_t param_result_;
  mojo::ScopedDataPipeConsumerHandle param_receive_stream_;
  mojo::ScopedDataPipeProducerHandle param_send_stream_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_CreateTCPConnectedSocket_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_CreateTCPConnectedSocket_Response_Message::kMessageTag = {};

bool NetworkContext_CreateTCPConnectedSocket_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::CreateTCPConnectedSocketCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_CreateTCPConnectedSocket_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_CreateTCPConnectedSocket_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_CreateTCPConnectedSocket_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  int32_t p_result{};
  mojo::ScopedDataPipeConsumerHandle p_receive_stream{};
  mojo::ScopedDataPipeProducerHandle p_send_stream{};
  NetworkContext_CreateTCPConnectedSocket_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_result = input_data_view.result();
  p_receive_stream = input_data_view.TakeReceiveStream();
  p_send_stream = input_data_view.TakeSendStream();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::CreateTCPConnectedSocket response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_result), 
std::move(p_receive_stream), 
std::move(p_send_stream));
  return true;
}

void NetworkContext_CreateTCPConnectedSocket_ProxyToResponder::Run(
    int32_t in_result, mojo::ScopedDataPipeConsumerHandle in_receive_stream, mojo::ScopedDataPipeProducerHandle in_send_stream) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_CreateTCPConnectedSocket_Response_Message::Build(kSerialize, is_sync_, std::move(in_result), std::move(in_receive_stream), std::move(in_send_stream));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateTCPConnectedSocketCallback",
               "message", message.name());
#endif
  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkContext_AddHSTSForTesting_ProxyToResponder {
 public:
  static NetworkContext::AddHSTSForTestingCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkContext_AddHSTSForTesting_ProxyToResponder> proxy(
        new NetworkContext_AddHSTSForTesting_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkContext_AddHSTSForTesting_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkContext_AddHSTSForTesting_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkContext_AddHSTSForTesting_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkContext::AddHSTSForTestingCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_AddHSTSForTesting_ProxyToResponder);
};
class NetworkContext_AddHSTSForTesting_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkContext_AddHSTSForTesting_Response_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkContext_AddHSTSForTesting_Name,
          message_flags){}
  ~NetworkContext_AddHSTSForTesting_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkContext_AddHSTSForTesting_Response_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkContext_AddHSTSForTesting_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkContext_AddHSTSForTesting_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkContext::AddHSTSForTestingCallback* callback) {
    std::move(*callback).Run();
  }


  void HandleSyncResponse(
) {

  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkContext_AddHSTSForTesting_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkContext_AddHSTSForTesting_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkContext_AddHSTSForTesting_Response_Message::kMessageTag = {};

bool NetworkContext_AddHSTSForTesting_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkContext::AddHSTSForTestingCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_AddHSTSForTesting_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_AddHSTSForTesting_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkContext_AddHSTSForTesting_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_AddHSTSForTesting_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::AddHSTSForTesting response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void NetworkContext_AddHSTSForTesting_ProxyToResponder::Run(
    ) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkContext_AddHSTSForTesting_Response_Message::Build(kSerialize, is_sync_);

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::AddHSTSForTestingCallback",
               "message", message.name());
#endif
  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
bool NetworkContext_AddHSTSForTesting_HandleSyncResponse::Accept(
    mojo::Message* message) {
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkContext_AddHSTSForTesting_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was built using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      context->HandleSyncResponse();
      *result_ = true;
      mojo::internal::SyncMessageResponseSetup::SetCurrentSyncResponseMessage(
          message);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkContext_AddHSTSForTesting_ResponseParams_Data* params =
      reinterpret_cast<internal::NetworkContext_AddHSTSForTesting_ResponseParams_Data*>(
          message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  NetworkContext_AddHSTSForTesting_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkContext::AddHSTSForTesting response deserializer");
    return false;
  }
  mojo::internal::SyncMessageResponseSetup::SetCurrentSyncResponseMessage(
      message);
  *result_ = true;
  return true;
}

// static
bool NetworkContextStubDispatch::Accept(
    NetworkContext* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kNetworkContext_CreateURLLoaderFactory_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateURLLoaderFactory",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_CreateURLLoaderFactory_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_CreateURLLoaderFactory_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_CreateURLLoaderFactory_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::network::mojom::URLLoaderFactoryRequest p_url_loader_factory{};
      uint32_t p_process_id{};
      NetworkContext_CreateURLLoaderFactory_ParamsDataView input_data_view(params, &serialization_context);
      
      p_url_loader_factory =
          input_data_view.TakeUrlLoaderFactory<decltype(p_url_loader_factory)>();
      p_process_id = input_data_view.process_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::CreateURLLoaderFactory deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateURLLoaderFactory(
std::move(p_url_loader_factory), 
std::move(p_process_id));
      return true;
    }
    case internal::kNetworkContext_GetCookieManager_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::GetCookieManager",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_GetCookieManager_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_GetCookieManager_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_GetCookieManager_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::network::mojom::CookieManagerRequest p_cookie_manager{};
      NetworkContext_GetCookieManager_ParamsDataView input_data_view(params, &serialization_context);
      
      p_cookie_manager =
          input_data_view.TakeCookieManager<decltype(p_cookie_manager)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::GetCookieManager deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetCookieManager(
std::move(p_cookie_manager));
      return true;
    }
    case internal::kNetworkContext_GetRestrictedCookieManager_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::GetRestrictedCookieManager",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_GetRestrictedCookieManager_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_GetRestrictedCookieManager_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_GetRestrictedCookieManager_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::network::mojom::RestrictedCookieManagerRequest p_restricted_cookie_manager{};
      int32_t p_render_process_id{};
      int32_t p_render_frame_id{};
      NetworkContext_GetRestrictedCookieManager_ParamsDataView input_data_view(params, &serialization_context);
      
      p_restricted_cookie_manager =
          input_data_view.TakeRestrictedCookieManager<decltype(p_restricted_cookie_manager)>();
      p_render_process_id = input_data_view.render_process_id();
      p_render_frame_id = input_data_view.render_frame_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::GetRestrictedCookieManager deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetRestrictedCookieManager(
std::move(p_restricted_cookie_manager), 
std::move(p_render_process_id), 
std::move(p_render_frame_id));
      return true;
    }
    case internal::kNetworkContext_ClearNetworkingHistorySince_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearNetworkingHistorySince",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_ClearHttpCache_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearHttpCache",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_SetNetworkConditions_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetNetworkConditions",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_SetNetworkConditions_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_SetNetworkConditions_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_SetNetworkConditions_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_profile_id{};
      NetworkConditionsPtr p_conditions{};
      NetworkContext_SetNetworkConditions_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadProfileId(&p_profile_id))
        success = false;
      if (!input_data_view.ReadConditions(&p_conditions))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::SetNetworkConditions deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetNetworkConditions(
std::move(p_profile_id), 
std::move(p_conditions));
      return true;
    }
    case internal::kNetworkContext_SetAcceptLanguage_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetAcceptLanguage",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_SetAcceptLanguage_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_SetAcceptLanguage_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_SetAcceptLanguage_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_new_accept_language{};
      NetworkContext_SetAcceptLanguage_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadNewAcceptLanguage(&p_new_accept_language))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::SetAcceptLanguage deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetAcceptLanguage(
std::move(p_new_accept_language));
      return true;
    }
    case internal::kNetworkContext_SetCTPolicy_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetCTPolicy",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_SetCTPolicy_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_SetCTPolicy_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_SetCTPolicy_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::vector<std::string> p_required_hosts{};
      std::vector<std::string> p_excluded_hosts{};
      std::vector<std::string> p_excluded_spkis{};
      std::vector<std::string> p_excluded_legacy_spkis{};
      NetworkContext_SetCTPolicy_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadRequiredHosts(&p_required_hosts))
        success = false;
      if (!input_data_view.ReadExcludedHosts(&p_excluded_hosts))
        success = false;
      if (!input_data_view.ReadExcludedSpkis(&p_excluded_spkis))
        success = false;
      if (!input_data_view.ReadExcludedLegacySpkis(&p_excluded_legacy_spkis))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::SetCTPolicy deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetCTPolicy(
std::move(p_required_hosts), 
std::move(p_excluded_hosts), 
std::move(p_excluded_spkis), 
std::move(p_excluded_legacy_spkis));
      return true;
    }
    case internal::kNetworkContext_CreateUDPSocket_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateUDPSocket",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_CreateUDPSocket_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_CreateUDPSocket_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_CreateUDPSocket_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::network::mojom::UDPSocketRequest p_request{};
      ::network::mojom::UDPSocketReceiverPtr p_receiver{};
      NetworkContext_CreateUDPSocket_ParamsDataView input_data_view(params, &serialization_context);
      
      p_request =
          input_data_view.TakeRequest<decltype(p_request)>();
      p_receiver =
          input_data_view.TakeReceiver<decltype(p_receiver)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::CreateUDPSocket deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateUDPSocket(
std::move(p_request), 
std::move(p_receiver));
      return true;
    }
    case internal::kNetworkContext_CreateTCPServerSocket_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateTCPServerSocket",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_CreateTCPConnectedSocket_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateTCPConnectedSocket",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_CreateWebSocket_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateWebSocket",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_CreateWebSocket_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkContext_CreateWebSocket_Params_Data* params =
          reinterpret_cast<internal::NetworkContext_CreateWebSocket_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::network::mojom::WebSocketRequest p_request{};
      int32_t p_process_id{};
      int32_t p_render_frame_id{};
      url::Origin p_origin{};
      NetworkContext_CreateWebSocket_ParamsDataView input_data_view(params, &serialization_context);
      
      p_request =
          input_data_view.TakeRequest<decltype(p_request)>();
      p_process_id = input_data_view.process_id();
      p_render_frame_id = input_data_view.render_frame_id();
      if (!input_data_view.ReadOrigin(&p_origin))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::CreateWebSocket deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateWebSocket(
std::move(p_request), 
std::move(p_process_id), 
std::move(p_render_frame_id), 
std::move(p_origin));
      return true;
    }
    case internal::kNetworkContext_AddHSTSForTesting_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::AddHSTSForTesting",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

// static
bool NetworkContextStubDispatch::AcceptWithResponder(
    NetworkContext* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kNetworkContext_CreateURLLoaderFactory_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateURLLoaderFactory",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_GetCookieManager_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::GetCookieManager",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_GetRestrictedCookieManager_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::GetRestrictedCookieManager",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_ClearNetworkingHistorySince_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearNetworkingHistorySince",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_ClearNetworkingHistorySince_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::ClearNetworkingHistorySinceCallback callback =
              NetworkContext_ClearNetworkingHistorySince_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_ClearNetworkingHistorySince_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_ClearNetworkingHistorySince_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      base::Time p_start_time{};
      NetworkContext_ClearNetworkingHistorySince_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadStartTime(&p_start_time))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::ClearNetworkingHistorySince deserializer");
        return false;
      }
      NetworkContext::ClearNetworkingHistorySinceCallback callback =
          NetworkContext_ClearNetworkingHistorySince_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ClearNetworkingHistorySince(
std::move(p_start_time), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_ClearHttpCache_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::ClearHttpCache",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_ClearHttpCache_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::ClearHttpCacheCallback callback =
              NetworkContext_ClearHttpCache_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_ClearHttpCache_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_ClearHttpCache_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      base::Time p_start_time{};
      base::Time p_end_time{};
      ClearCacheUrlFilterPtr p_filter{};
      NetworkContext_ClearHttpCache_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadStartTime(&p_start_time))
        success = false;
      if (!input_data_view.ReadEndTime(&p_end_time))
        success = false;
      if (!input_data_view.ReadFilter(&p_filter))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::ClearHttpCache deserializer");
        return false;
      }
      NetworkContext::ClearHttpCacheCallback callback =
          NetworkContext_ClearHttpCache_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ClearHttpCache(
std::move(p_start_time), 
std::move(p_end_time), 
std::move(p_filter), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_SetNetworkConditions_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetNetworkConditions",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_SetAcceptLanguage_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetAcceptLanguage",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_SetCTPolicy_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::SetCTPolicy",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_CreateUDPSocket_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateUDPSocket",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_CreateTCPServerSocket_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateTCPServerSocket",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_CreateTCPServerSocket_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::CreateTCPServerSocketCallback callback =
              NetworkContext_CreateTCPServerSocket_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_CreateTCPServerSocket_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_CreateTCPServerSocket_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      net::IPEndPoint p_local_addr{};
      uint32_t p_backlog{};
      net::MutableNetworkTrafficAnnotationTag p_traffic_annotation{};
      ::network::mojom::TCPServerSocketRequest p_socket{};
      NetworkContext_CreateTCPServerSocket_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadLocalAddr(&p_local_addr))
        success = false;
      p_backlog = input_data_view.backlog();
      if (!input_data_view.ReadTrafficAnnotation(&p_traffic_annotation))
        success = false;
      p_socket =
          input_data_view.TakeSocket<decltype(p_socket)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::CreateTCPServerSocket deserializer");
        return false;
      }
      NetworkContext::CreateTCPServerSocketCallback callback =
          NetworkContext_CreateTCPServerSocket_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateTCPServerSocket(
std::move(p_local_addr), 
std::move(p_backlog), 
std::move(p_traffic_annotation), 
std::move(p_socket), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_CreateTCPConnectedSocket_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateTCPConnectedSocket",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_CreateTCPConnectedSocket_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::CreateTCPConnectedSocketCallback callback =
              NetworkContext_CreateTCPConnectedSocket_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_CreateTCPConnectedSocket_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_CreateTCPConnectedSocket_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      base::Optional<net::IPEndPoint> p_local_addr{};
      net::AddressList p_remote_addr_list{};
      net::MutableNetworkTrafficAnnotationTag p_traffic_annotation{};
      ::network::mojom::TCPConnectedSocketRequest p_socket{};
      ::network::mojom::TCPConnectedSocketObserverPtr p_observer{};
      NetworkContext_CreateTCPConnectedSocket_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadLocalAddr(&p_local_addr))
        success = false;
      if (!input_data_view.ReadRemoteAddrList(&p_remote_addr_list))
        success = false;
      if (!input_data_view.ReadTrafficAnnotation(&p_traffic_annotation))
        success = false;
      p_socket =
          input_data_view.TakeSocket<decltype(p_socket)>();
      p_observer =
          input_data_view.TakeObserver<decltype(p_observer)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::CreateTCPConnectedSocket deserializer");
        return false;
      }
      NetworkContext::CreateTCPConnectedSocketCallback callback =
          NetworkContext_CreateTCPConnectedSocket_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateTCPConnectedSocket(
std::move(p_local_addr), 
std::move(p_remote_addr_list), 
std::move(p_traffic_annotation), 
std::move(p_socket), 
std::move(p_observer), std::move(callback));
      return true;
    }
    case internal::kNetworkContext_CreateWebSocket_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::CreateWebSocket",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkContext_AddHSTSForTesting_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkContext::AddHSTSForTesting",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkContextProxy_AddHSTSForTesting_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkContext::AddHSTSForTestingCallback callback =
              NetworkContext_AddHSTSForTesting_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkContext_AddHSTSForTesting_Params_Data* params =
          reinterpret_cast<
              internal::NetworkContext_AddHSTSForTesting_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_host{};
      base::Time p_expiry{};
      bool p_include_subdomains{};
      NetworkContext_AddHSTSForTesting_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadHost(&p_host))
        success = false;
      if (!input_data_view.ReadExpiry(&p_expiry))
        success = false;
      p_include_subdomains = input_data_view.include_subdomains();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkContext::AddHSTSForTesting deserializer");
        return false;
      }
      NetworkContext::AddHSTSForTestingCallback callback =
          NetworkContext_AddHSTSForTesting_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->AddHSTSForTesting(
std::move(p_host), 
std::move(p_expiry), 
std::move(p_include_subdomains), std::move(callback));
      return true;
    }
  }
  return false;
}

bool NetworkContextRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "NetworkContext RequestValidator");

  switch (message->header()->name) {
    case internal::kNetworkContext_CreateURLLoaderFactory_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CreateURLLoaderFactory_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_GetCookieManager_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_GetCookieManager_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_GetRestrictedCookieManager_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_GetRestrictedCookieManager_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearNetworkingHistorySince_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearNetworkingHistorySince_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearHttpCache_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearHttpCache_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_SetNetworkConditions_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_SetNetworkConditions_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_SetAcceptLanguage_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_SetAcceptLanguage_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_SetCTPolicy_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_SetCTPolicy_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CreateUDPSocket_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CreateUDPSocket_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CreateTCPServerSocket_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CreateTCPServerSocket_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CreateTCPConnectedSocket_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CreateTCPConnectedSocket_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CreateWebSocket_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CreateWebSocket_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_AddHSTSForTesting_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_AddHSTSForTesting_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool NetworkContextResponseValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "NetworkContext ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kNetworkContext_ClearNetworkingHistorySince_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearNetworkingHistorySince_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_ClearHttpCache_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_ClearHttpCache_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CreateTCPServerSocket_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CreateTCPServerSocket_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_CreateTCPConnectedSocket_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_CreateTCPConnectedSocket_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkContext_AddHSTSForTesting_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkContext_AddHSTSForTesting_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
void NetworkContextInterceptorForTesting::CreateURLLoaderFactory(::network::mojom::URLLoaderFactoryRequest url_loader_factory, uint32_t process_id) {
  GetForwardingInterface()->CreateURLLoaderFactory(std::move(url_loader_factory), std::move(process_id));
}
void NetworkContextInterceptorForTesting::GetCookieManager(::network::mojom::CookieManagerRequest cookie_manager) {
  GetForwardingInterface()->GetCookieManager(std::move(cookie_manager));
}
void NetworkContextInterceptorForTesting::GetRestrictedCookieManager(::network::mojom::RestrictedCookieManagerRequest restricted_cookie_manager, int32_t render_process_id, int32_t render_frame_id) {
  GetForwardingInterface()->GetRestrictedCookieManager(std::move(restricted_cookie_manager), std::move(render_process_id), std::move(render_frame_id));
}
void NetworkContextInterceptorForTesting::ClearNetworkingHistorySince(base::Time start_time, ClearNetworkingHistorySinceCallback callback) {
  GetForwardingInterface()->ClearNetworkingHistorySince(std::move(start_time), std::move(callback));
}
void NetworkContextInterceptorForTesting::ClearHttpCache(base::Time start_time, base::Time end_time, ClearCacheUrlFilterPtr filter, ClearHttpCacheCallback callback) {
  GetForwardingInterface()->ClearHttpCache(std::move(start_time), std::move(end_time), std::move(filter), std::move(callback));
}
void NetworkContextInterceptorForTesting::SetNetworkConditions(const std::string& profile_id, NetworkConditionsPtr conditions) {
  GetForwardingInterface()->SetNetworkConditions(std::move(profile_id), std::move(conditions));
}
void NetworkContextInterceptorForTesting::SetAcceptLanguage(const std::string& new_accept_language) {
  GetForwardingInterface()->SetAcceptLanguage(std::move(new_accept_language));
}
void NetworkContextInterceptorForTesting::SetCTPolicy(const std::vector<std::string>& required_hosts, const std::vector<std::string>& excluded_hosts, const std::vector<std::string>& excluded_spkis, const std::vector<std::string>& excluded_legacy_spkis) {
  GetForwardingInterface()->SetCTPolicy(std::move(required_hosts), std::move(excluded_hosts), std::move(excluded_spkis), std::move(excluded_legacy_spkis));
}
void NetworkContextInterceptorForTesting::CreateUDPSocket(::network::mojom::UDPSocketRequest request, ::network::mojom::UDPSocketReceiverPtr receiver) {
  GetForwardingInterface()->CreateUDPSocket(std::move(request), std::move(receiver));
}
void NetworkContextInterceptorForTesting::CreateTCPServerSocket(const net::IPEndPoint& local_addr, uint32_t backlog, const net::MutableNetworkTrafficAnnotationTag& traffic_annotation, ::network::mojom::TCPServerSocketRequest socket, CreateTCPServerSocketCallback callback) {
  GetForwardingInterface()->CreateTCPServerSocket(std::move(local_addr), std::move(backlog), std::move(traffic_annotation), std::move(socket), std::move(callback));
}
void NetworkContextInterceptorForTesting::CreateTCPConnectedSocket(const base::Optional<net::IPEndPoint>& local_addr, const net::AddressList& remote_addr_list, const net::MutableNetworkTrafficAnnotationTag& traffic_annotation, ::network::mojom::TCPConnectedSocketRequest socket, ::network::mojom::TCPConnectedSocketObserverPtr observer, CreateTCPConnectedSocketCallback callback) {
  GetForwardingInterface()->CreateTCPConnectedSocket(std::move(local_addr), std::move(remote_addr_list), std::move(traffic_annotation), std::move(socket), std::move(observer), std::move(callback));
}
void NetworkContextInterceptorForTesting::CreateWebSocket(::network::mojom::WebSocketRequest request, int32_t process_id, int32_t render_frame_id, const url::Origin& origin) {
  GetForwardingInterface()->CreateWebSocket(std::move(request), std::move(process_id), std::move(render_frame_id), std::move(origin));
}
void NetworkContextInterceptorForTesting::AddHSTSForTesting(const std::string& host, base::Time expiry, bool include_subdomains, AddHSTSForTestingCallback callback) {
  GetForwardingInterface()->AddHSTSForTesting(std::move(host), std::move(expiry), std::move(include_subdomains), std::move(callback));
}
NetworkContextAsyncWaiter::NetworkContextAsyncWaiter(
    NetworkContext* proxy) : proxy_(proxy) {}

NetworkContextAsyncWaiter::~NetworkContextAsyncWaiter() = default;

void NetworkContextAsyncWaiter::ClearNetworkingHistorySince(
    base::Time start_time) {
  base::RunLoop loop;
  proxy_->ClearNetworkingHistorySince(std::move(start_time),
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}
void NetworkContextAsyncWaiter::ClearHttpCache(
    base::Time start_time, base::Time end_time, ClearCacheUrlFilterPtr filter) {
  base::RunLoop loop;
  proxy_->ClearHttpCache(std::move(start_time),std::move(end_time),std::move(filter),
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}
void NetworkContextAsyncWaiter::CreateTCPServerSocket(
    const net::IPEndPoint& local_addr, uint32_t backlog, const net::MutableNetworkTrafficAnnotationTag& traffic_annotation, ::network::mojom::TCPServerSocketRequest socket, int32_t* out_result, base::Optional<net::IPEndPoint>* out_local_addr_out) {
  base::RunLoop loop;
  proxy_->CreateTCPServerSocket(std::move(local_addr),std::move(backlog),std::move(traffic_annotation),std::move(socket),
      base::BindOnce(
          [](base::RunLoop* loop,
             int32_t* out_result
,
             base::Optional<net::IPEndPoint>* out_local_addr_out
,
             int32_t result,
             const base::Optional<net::IPEndPoint>& local_addr_out) {*out_result = std::move(result);*out_local_addr_out = std::move(local_addr_out);
            loop->Quit();
          },
          &loop,
          out_result,
          out_local_addr_out));
  loop.Run();
}
void NetworkContextAsyncWaiter::CreateTCPConnectedSocket(
    const base::Optional<net::IPEndPoint>& local_addr, const net::AddressList& remote_addr_list, const net::MutableNetworkTrafficAnnotationTag& traffic_annotation, ::network::mojom::TCPConnectedSocketRequest socket, ::network::mojom::TCPConnectedSocketObserverPtr observer, int32_t* out_result, mojo::ScopedDataPipeConsumerHandle* out_receive_stream, mojo::ScopedDataPipeProducerHandle* out_send_stream) {
  base::RunLoop loop;
  proxy_->CreateTCPConnectedSocket(std::move(local_addr),std::move(remote_addr_list),std::move(traffic_annotation),std::move(socket),std::move(observer),
      base::BindOnce(
          [](base::RunLoop* loop,
             int32_t* out_result
,
             mojo::ScopedDataPipeConsumerHandle* out_receive_stream
,
             mojo::ScopedDataPipeProducerHandle* out_send_stream
,
             int32_t result,
             mojo::ScopedDataPipeConsumerHandle receive_stream,
             mojo::ScopedDataPipeProducerHandle send_stream) {*out_result = std::move(result);*out_receive_stream = std::move(receive_stream);*out_send_stream = std::move(send_stream);
            loop->Quit();
          },
          &loop,
          out_result,
          out_receive_stream,
          out_send_stream));
  loop.Run();
}
void NetworkContextAsyncWaiter::AddHSTSForTesting(
    const std::string& host, base::Time expiry, bool include_subdomains) {
  base::RunLoop loop;
  proxy_->AddHSTSForTesting(std::move(host),std::move(expiry),std::move(include_subdomains),
      base::BindOnce(
          [](base::RunLoop* loop) {
            loop->Quit();
          },
          &loop));
  loop.Run();
}

const char SSLPrivateKey::Name_[] = "network::mojom::SSLPrivateKey";

class SSLPrivateKey_Sign_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  SSLPrivateKey_Sign_ForwardToCallback(
      SSLPrivateKey::SignCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  SSLPrivateKey::SignCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(SSLPrivateKey_Sign_ForwardToCallback);
};

SSLPrivateKeyProxy::SSLPrivateKeyProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}
class SSLPrivateKeyProxy_Sign_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit SSLPrivateKeyProxy_Sign_Message(
      uint32_t message_flags
      , uint16_t param_algorithm
      , const std::vector<uint8_t>& param_input
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kSSLPrivateKey_Sign_Name,
          message_flags)
      , param_algorithm_(std::move(param_algorithm))
      , param_input_(std::move(param_input)){}
  ~SSLPrivateKeyProxy_Sign_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      uint16_t param_algorithm, const std::vector<uint8_t>& param_input) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<SSLPrivateKeyProxy_Sign_Message>(
          kFlags
          , std::move(param_algorithm)
          , std::move(param_input)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kSSLPrivateKey_Sign_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::SSLPrivateKey_Sign_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->algorithm = param_algorithm;
  typename decltype(params->input)::BaseType::BufferWriter
      input_writer;
  const mojo::internal::ContainerValidateParams input_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
      param_input, buffer, &input_writer, &input_validate_params,
      &serialization_context);
  params->input.Set(
      input_writer.is_null() ? nullptr : input_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->input.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null input in SSLPrivateKey.Sign request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(SSLPrivateKey* impl, SSLPrivateKey::SignCallback callback) {
    impl->Sign(
        std::move(param_algorithm_),
        std::move(param_input_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::SSLPrivateKey_Sign_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->algorithm = param_algorithm_;
  typename decltype(params->input)::BaseType::BufferWriter
      input_writer;
  const mojo::internal::ContainerValidateParams input_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
      param_input_, buffer, &input_writer, &input_validate_params,
      serialization_context);
  params->input.Set(
      input_writer.is_null() ? nullptr : input_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->input.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null input in SSLPrivateKey.Sign request");
  }
  uint16_t param_algorithm_;
  std::vector<uint8_t> param_input_;

  DISALLOW_COPY_AND_ASSIGN(SSLPrivateKeyProxy_Sign_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
SSLPrivateKeyProxy_Sign_Message::kMessageTag = {};

void SSLPrivateKeyProxy::Sign(
    uint16_t in_algorithm, const std::vector<uint8_t>& in_input, SignCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::SSLPrivateKey::Sign");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = SSLPrivateKeyProxy_Sign_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_algorithm), std::move(in_input));
  std::unique_ptr<mojo::MessageReceiver> responder(
      new SSLPrivateKey_Sign_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class SSLPrivateKey_Sign_ProxyToResponder {
 public:
  static SSLPrivateKey::SignCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<SSLPrivateKey_Sign_ProxyToResponder> proxy(
        new SSLPrivateKey_Sign_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&SSLPrivateKey_Sign_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~SSLPrivateKey_Sign_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  SSLPrivateKey_Sign_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "SSLPrivateKey::SignCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      int32_t in_net_error, const std::vector<uint8_t>& in_signature);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(SSLPrivateKey_Sign_ProxyToResponder);
};
class SSLPrivateKey_Sign_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit SSLPrivateKey_Sign_Response_Message(
      uint32_t message_flags
      , int32_t param_net_error
      , const std::vector<uint8_t>& param_signature
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kSSLPrivateKey_Sign_Name,
          message_flags)
      , param_net_error_(std::move(param_net_error))
      , param_signature_(std::move(param_signature)){}
  ~SSLPrivateKey_Sign_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      int32_t param_net_error, const std::vector<uint8_t>& param_signature) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<SSLPrivateKey_Sign_Response_Message>(
          kFlags
          , std::move(param_net_error)
          , std::move(param_signature)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kSSLPrivateKey_Sign_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::SSLPrivateKey_Sign_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->net_error = param_net_error;
  typename decltype(params->signature)::BaseType::BufferWriter
      signature_writer;
  const mojo::internal::ContainerValidateParams signature_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
      param_signature, buffer, &signature_writer, &signature_validate_params,
      &serialization_context);
  params->signature.Set(
      signature_writer.is_null() ? nullptr : signature_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->signature.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null signature in SSLPrivateKey.Sign response");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(SSLPrivateKey::SignCallback* callback) {
    std::move(*callback).Run(
        std::move(param_net_error_), 
        std::move(param_signature_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::SSLPrivateKey_Sign_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->net_error = param_net_error_;
  typename decltype(params->signature)::BaseType::BufferWriter
      signature_writer;
  const mojo::internal::ContainerValidateParams signature_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
      param_signature_, buffer, &signature_writer, &signature_validate_params,
      serialization_context);
  params->signature.Set(
      signature_writer.is_null() ? nullptr : signature_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->signature.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null signature in SSLPrivateKey.Sign response");
  }
  int32_t param_net_error_;
  std::vector<uint8_t> param_signature_;

  DISALLOW_COPY_AND_ASSIGN(SSLPrivateKey_Sign_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
SSLPrivateKey_Sign_Response_Message::kMessageTag = {};

bool SSLPrivateKey_Sign_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::SSLPrivateKey::SignCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<SSLPrivateKey_Sign_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::SSLPrivateKey_Sign_ResponseParams_Data* params =
      reinterpret_cast<
          internal::SSLPrivateKey_Sign_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  int32_t p_net_error{};
  std::vector<uint8_t> p_signature{};
  SSLPrivateKey_Sign_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_net_error = input_data_view.net_error();
  if (!input_data_view.ReadSignature(&p_signature))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "SSLPrivateKey::Sign response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_net_error), 
std::move(p_signature));
  return true;
}

void SSLPrivateKey_Sign_ProxyToResponder::Run(
    int32_t in_net_error, const std::vector<uint8_t>& in_signature) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = SSLPrivateKey_Sign_Response_Message::Build(kSerialize, is_sync_, std::move(in_net_error), std::move(in_signature));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::SSLPrivateKey::SignCallback",
               "message", message.name());
#endif
  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool SSLPrivateKeyStubDispatch::Accept(
    SSLPrivateKey* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kSSLPrivateKey_Sign_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::SSLPrivateKey::Sign",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

// static
bool SSLPrivateKeyStubDispatch::AcceptWithResponder(
    SSLPrivateKey* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kSSLPrivateKey_Sign_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::SSLPrivateKey::Sign",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            SSLPrivateKeyProxy_Sign_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          SSLPrivateKey::SignCallback callback =
              SSLPrivateKey_Sign_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::SSLPrivateKey_Sign_Params_Data* params =
          reinterpret_cast<
              internal::SSLPrivateKey_Sign_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint16_t p_algorithm{};
      std::vector<uint8_t> p_input{};
      SSLPrivateKey_Sign_ParamsDataView input_data_view(params, &serialization_context);
      
      p_algorithm = input_data_view.algorithm();
      if (!input_data_view.ReadInput(&p_input))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "SSLPrivateKey::Sign deserializer");
        return false;
      }
      SSLPrivateKey::SignCallback callback =
          SSLPrivateKey_Sign_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->Sign(
std::move(p_algorithm), 
std::move(p_input), std::move(callback));
      return true;
    }
  }
  return false;
}

bool SSLPrivateKeyRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "SSLPrivateKey RequestValidator");

  switch (message->header()->name) {
    case internal::kSSLPrivateKey_Sign_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::SSLPrivateKey_Sign_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool SSLPrivateKeyResponseValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "SSLPrivateKey ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kSSLPrivateKey_Sign_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::SSLPrivateKey_Sign_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
void SSLPrivateKeyInterceptorForTesting::Sign(uint16_t algorithm, const std::vector<uint8_t>& input, SignCallback callback) {
  GetForwardingInterface()->Sign(std::move(algorithm), std::move(input), std::move(callback));
}
SSLPrivateKeyAsyncWaiter::SSLPrivateKeyAsyncWaiter(
    SSLPrivateKey* proxy) : proxy_(proxy) {}

SSLPrivateKeyAsyncWaiter::~SSLPrivateKeyAsyncWaiter() = default;

void SSLPrivateKeyAsyncWaiter::Sign(
    uint16_t algorithm, const std::vector<uint8_t>& input, int32_t* out_net_error, std::vector<uint8_t>* out_signature) {
  base::RunLoop loop;
  proxy_->Sign(std::move(algorithm),std::move(input),
      base::BindOnce(
          [](base::RunLoop* loop,
             int32_t* out_net_error
,
             std::vector<uint8_t>* out_signature
,
             int32_t net_error,
             const std::vector<uint8_t>& signature) {*out_net_error = std::move(net_error);*out_signature = std::move(signature);
            loop->Quit();
          },
          &loop,
          out_net_error,
          out_signature));
  loop.Run();
}

const char NetworkServiceClient::Name_[] = "network::mojom::NetworkServiceClient";

class NetworkServiceClient_OnAuthRequired_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkServiceClient_OnAuthRequired_ForwardToCallback(
      NetworkServiceClient::OnAuthRequiredCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkServiceClient::OnAuthRequiredCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClient_OnAuthRequired_ForwardToCallback);
};

class NetworkServiceClient_OnCertificateRequested_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkServiceClient_OnCertificateRequested_ForwardToCallback(
      NetworkServiceClient::OnCertificateRequestedCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkServiceClient::OnCertificateRequestedCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClient_OnCertificateRequested_ForwardToCallback);
};

class NetworkServiceClient_OnSSLCertificateError_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  NetworkServiceClient_OnSSLCertificateError_ForwardToCallback(
      NetworkServiceClient::OnSSLCertificateErrorCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  NetworkServiceClient::OnSSLCertificateErrorCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClient_OnSSLCertificateError_ForwardToCallback);
};

NetworkServiceClientProxy::NetworkServiceClientProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}
class NetworkServiceClientProxy_OnAuthRequired_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceClientProxy_OnAuthRequired_Message(
      uint32_t message_flags
      , uint32_t param_process_id
      , uint32_t param_routing_id
      , uint32_t param_request_id
      , int32_t param_resource_type
      , const GURL& param_url
      , bool param_first_auth_attempt
      , const scoped_refptr<net::AuthChallengeInfo>& param_auth_info
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkServiceClient_OnAuthRequired_Name,
          message_flags)
      , param_process_id_(std::move(param_process_id))
      , param_routing_id_(std::move(param_routing_id))
      , param_request_id_(std::move(param_request_id))
      , param_resource_type_(std::move(param_resource_type))
      , param_url_(std::move(param_url))
      , param_first_auth_attempt_(std::move(param_first_auth_attempt))
      , param_auth_info_(std::move(param_auth_info)){}
  ~NetworkServiceClientProxy_OnAuthRequired_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      uint32_t param_process_id, uint32_t param_routing_id, uint32_t param_request_id, int32_t param_resource_type, const GURL& param_url, bool param_first_auth_attempt, const scoped_refptr<net::AuthChallengeInfo>& param_auth_info) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceClientProxy_OnAuthRequired_Message>(
          kFlags
          , std::move(param_process_id)
          , std::move(param_routing_id)
          , std::move(param_request_id)
          , std::move(param_resource_type)
          , std::move(param_url)
          , std::move(param_first_auth_attempt)
          , std::move(param_auth_info)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkServiceClient_OnAuthRequired_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkServiceClient_OnAuthRequired_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->process_id = param_process_id;
  params->routing_id = param_routing_id;
  params->request_id = param_request_id;
  params->resource_type = param_resource_type;
  typename decltype(params->url)::BaseType::BufferWriter
      url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_url, buffer, &url_writer, &serialization_context);
  params->url.Set(
      url_writer.is_null() ? nullptr : url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in NetworkServiceClient.OnAuthRequired request");
  params->first_auth_attempt = param_first_auth_attempt;
  typename decltype(params->auth_info)::BaseType::BufferWriter
      auth_info_writer;
  mojo::internal::Serialize<::network::mojom::AuthChallengeInfoDataView>(
      param_auth_info, buffer, &auth_info_writer, &serialization_context);
  params->auth_info.Set(
      auth_info_writer.is_null() ? nullptr : auth_info_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->auth_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null auth_info in NetworkServiceClient.OnAuthRequired request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkServiceClient* impl, NetworkServiceClient::OnAuthRequiredCallback callback) {
    impl->OnAuthRequired(
        std::move(param_process_id_),
        std::move(param_routing_id_),
        std::move(param_request_id_),
        std::move(param_resource_type_),
        std::move(param_url_),
        std::move(param_first_auth_attempt_),
        std::move(param_auth_info_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkServiceClient_OnAuthRequired_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->process_id = param_process_id_;
  params->routing_id = param_routing_id_;
  params->request_id = param_request_id_;
  params->resource_type = param_resource_type_;
  typename decltype(params->url)::BaseType::BufferWriter
      url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_url_, buffer, &url_writer, serialization_context);
  params->url.Set(
      url_writer.is_null() ? nullptr : url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in NetworkServiceClient.OnAuthRequired request");
  params->first_auth_attempt = param_first_auth_attempt_;
  typename decltype(params->auth_info)::BaseType::BufferWriter
      auth_info_writer;
  mojo::internal::Serialize<::network::mojom::AuthChallengeInfoDataView>(
      param_auth_info_, buffer, &auth_info_writer, serialization_context);
  params->auth_info.Set(
      auth_info_writer.is_null() ? nullptr : auth_info_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->auth_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null auth_info in NetworkServiceClient.OnAuthRequired request");
  }
  uint32_t param_process_id_;
  uint32_t param_routing_id_;
  uint32_t param_request_id_;
  int32_t param_resource_type_;
  GURL param_url_;
  bool param_first_auth_attempt_;
  scoped_refptr<net::AuthChallengeInfo> param_auth_info_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClientProxy_OnAuthRequired_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceClientProxy_OnAuthRequired_Message::kMessageTag = {};

void NetworkServiceClientProxy::OnAuthRequired(
    uint32_t in_process_id, uint32_t in_routing_id, uint32_t in_request_id, int32_t in_resource_type, const GURL& in_url, bool in_first_auth_attempt, const scoped_refptr<net::AuthChallengeInfo>& in_auth_info, OnAuthRequiredCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkServiceClient::OnAuthRequired");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkServiceClientProxy_OnAuthRequired_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_process_id), std::move(in_routing_id), std::move(in_request_id), std::move(in_resource_type), std::move(in_url), std::move(in_first_auth_attempt), std::move(in_auth_info));
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkServiceClient_OnAuthRequired_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkServiceClientProxy_OnCertificateRequested_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceClientProxy_OnCertificateRequested_Message(
      uint32_t message_flags
      , uint32_t param_process_id
      , uint32_t param_routing_id
      , uint32_t param_request_id
      , const scoped_refptr<net::SSLCertRequestInfo>& param_cert_info
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkServiceClient_OnCertificateRequested_Name,
          message_flags)
      , param_process_id_(std::move(param_process_id))
      , param_routing_id_(std::move(param_routing_id))
      , param_request_id_(std::move(param_request_id))
      , param_cert_info_(std::move(param_cert_info)){}
  ~NetworkServiceClientProxy_OnCertificateRequested_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      uint32_t param_process_id, uint32_t param_routing_id, uint32_t param_request_id, const scoped_refptr<net::SSLCertRequestInfo>& param_cert_info) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceClientProxy_OnCertificateRequested_Message>(
          kFlags
          , std::move(param_process_id)
          , std::move(param_routing_id)
          , std::move(param_request_id)
          , std::move(param_cert_info)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkServiceClient_OnCertificateRequested_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkServiceClient_OnCertificateRequested_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->process_id = param_process_id;
  params->routing_id = param_routing_id;
  params->request_id = param_request_id;
  typename decltype(params->cert_info)::BaseType::BufferWriter
      cert_info_writer;
  mojo::internal::Serialize<::network::mojom::SSLCertRequestInfoDataView>(
      param_cert_info, buffer, &cert_info_writer, &serialization_context);
  params->cert_info.Set(
      cert_info_writer.is_null() ? nullptr : cert_info_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->cert_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null cert_info in NetworkServiceClient.OnCertificateRequested request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkServiceClient* impl, NetworkServiceClient::OnCertificateRequestedCallback callback) {
    impl->OnCertificateRequested(
        std::move(param_process_id_),
        std::move(param_routing_id_),
        std::move(param_request_id_),
        std::move(param_cert_info_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkServiceClient_OnCertificateRequested_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->process_id = param_process_id_;
  params->routing_id = param_routing_id_;
  params->request_id = param_request_id_;
  typename decltype(params->cert_info)::BaseType::BufferWriter
      cert_info_writer;
  mojo::internal::Serialize<::network::mojom::SSLCertRequestInfoDataView>(
      param_cert_info_, buffer, &cert_info_writer, serialization_context);
  params->cert_info.Set(
      cert_info_writer.is_null() ? nullptr : cert_info_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->cert_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null cert_info in NetworkServiceClient.OnCertificateRequested request");
  }
  uint32_t param_process_id_;
  uint32_t param_routing_id_;
  uint32_t param_request_id_;
  scoped_refptr<net::SSLCertRequestInfo> param_cert_info_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClientProxy_OnCertificateRequested_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceClientProxy_OnCertificateRequested_Message::kMessageTag = {};

void NetworkServiceClientProxy::OnCertificateRequested(
    uint32_t in_process_id, uint32_t in_routing_id, uint32_t in_request_id, const scoped_refptr<net::SSLCertRequestInfo>& in_cert_info, OnCertificateRequestedCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkServiceClient::OnCertificateRequested");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkServiceClientProxy_OnCertificateRequested_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_process_id), std::move(in_routing_id), std::move(in_request_id), std::move(in_cert_info));
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkServiceClient_OnCertificateRequested_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkServiceClientProxy_OnSSLCertificateError_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceClientProxy_OnSSLCertificateError_Message(
      uint32_t message_flags
      , uint32_t param_process_id
      , uint32_t param_routing_id
      , uint32_t param_request_id
      , int32_t param_resource_type
      , const GURL& param_url
      , const net::SSLInfo& param_ssl_info
      , bool param_fatal
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkServiceClient_OnSSLCertificateError_Name,
          message_flags)
      , param_process_id_(std::move(param_process_id))
      , param_routing_id_(std::move(param_routing_id))
      , param_request_id_(std::move(param_request_id))
      , param_resource_type_(std::move(param_resource_type))
      , param_url_(std::move(param_url))
      , param_ssl_info_(std::move(param_ssl_info))
      , param_fatal_(std::move(param_fatal)){}
  ~NetworkServiceClientProxy_OnSSLCertificateError_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      uint32_t param_process_id, uint32_t param_routing_id, uint32_t param_request_id, int32_t param_resource_type, const GURL& param_url, const net::SSLInfo& param_ssl_info, bool param_fatal) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceClientProxy_OnSSLCertificateError_Message>(
          kFlags
          , std::move(param_process_id)
          , std::move(param_routing_id)
          , std::move(param_request_id)
          , std::move(param_resource_type)
          , std::move(param_url)
          , std::move(param_ssl_info)
          , std::move(param_fatal)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkServiceClient_OnSSLCertificateError_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkServiceClient_OnSSLCertificateError_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->process_id = param_process_id;
  params->routing_id = param_routing_id;
  params->request_id = param_request_id;
  params->resource_type = param_resource_type;
  typename decltype(params->url)::BaseType::BufferWriter
      url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_url, buffer, &url_writer, &serialization_context);
  params->url.Set(
      url_writer.is_null() ? nullptr : url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in NetworkServiceClient.OnSSLCertificateError request");
  typename decltype(params->ssl_info)::BaseType::BufferWriter
      ssl_info_writer;
  mojo::internal::Serialize<::network::mojom::SSLInfoDataView>(
      param_ssl_info, buffer, &ssl_info_writer, &serialization_context);
  params->ssl_info.Set(
      ssl_info_writer.is_null() ? nullptr : ssl_info_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->ssl_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null ssl_info in NetworkServiceClient.OnSSLCertificateError request");
  params->fatal = param_fatal;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkServiceClient* impl, NetworkServiceClient::OnSSLCertificateErrorCallback callback) {
    impl->OnSSLCertificateError(
        std::move(param_process_id_),
        std::move(param_routing_id_),
        std::move(param_request_id_),
        std::move(param_resource_type_),
        std::move(param_url_),
        std::move(param_ssl_info_),
        std::move(param_fatal_), std::move(callback));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkServiceClient_OnSSLCertificateError_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->process_id = param_process_id_;
  params->routing_id = param_routing_id_;
  params->request_id = param_request_id_;
  params->resource_type = param_resource_type_;
  typename decltype(params->url)::BaseType::BufferWriter
      url_writer;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      param_url_, buffer, &url_writer, serialization_context);
  params->url.Set(
      url_writer.is_null() ? nullptr : url_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in NetworkServiceClient.OnSSLCertificateError request");
  typename decltype(params->ssl_info)::BaseType::BufferWriter
      ssl_info_writer;
  mojo::internal::Serialize<::network::mojom::SSLInfoDataView>(
      param_ssl_info_, buffer, &ssl_info_writer, serialization_context);
  params->ssl_info.Set(
      ssl_info_writer.is_null() ? nullptr : ssl_info_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->ssl_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null ssl_info in NetworkServiceClient.OnSSLCertificateError request");
  params->fatal = param_fatal_;
  }
  uint32_t param_process_id_;
  uint32_t param_routing_id_;
  uint32_t param_request_id_;
  int32_t param_resource_type_;
  GURL param_url_;
  net::SSLInfo param_ssl_info_;
  bool param_fatal_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClientProxy_OnSSLCertificateError_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceClientProxy_OnSSLCertificateError_Message::kMessageTag = {};

void NetworkServiceClientProxy::OnSSLCertificateError(
    uint32_t in_process_id, uint32_t in_routing_id, uint32_t in_request_id, int32_t in_resource_type, const GURL& in_url, const net::SSLInfo& in_ssl_info, bool in_fatal, OnSSLCertificateErrorCallback callback) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkServiceClient::OnSSLCertificateError");
#endif
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkServiceClientProxy_OnSSLCertificateError_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_process_id), std::move(in_routing_id), std::move(in_request_id), std::move(in_resource_type), std::move(in_url), std::move(in_ssl_info), std::move(in_fatal));
  std::unique_ptr<mojo::MessageReceiver> responder(
      new NetworkServiceClient_OnSSLCertificateError_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class NetworkServiceClient_OnAuthRequired_ProxyToResponder {
 public:
  static NetworkServiceClient::OnAuthRequiredCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkServiceClient_OnAuthRequired_ProxyToResponder> proxy(
        new NetworkServiceClient_OnAuthRequired_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkServiceClient_OnAuthRequired_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkServiceClient_OnAuthRequired_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkServiceClient_OnAuthRequired_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkServiceClient::OnAuthRequiredCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      const base::Optional<net::AuthCredentials>& in_credentials);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClient_OnAuthRequired_ProxyToResponder);
};
class NetworkServiceClient_OnAuthRequired_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceClient_OnAuthRequired_Response_Message(
      uint32_t message_flags
      , const base::Optional<net::AuthCredentials>& param_credentials
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkServiceClient_OnAuthRequired_Name,
          message_flags)
      , param_credentials_(std::move(param_credentials)){}
  ~NetworkServiceClient_OnAuthRequired_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      const base::Optional<net::AuthCredentials>& param_credentials) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceClient_OnAuthRequired_Response_Message>(
          kFlags
          , std::move(param_credentials)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkServiceClient_OnAuthRequired_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkServiceClient_OnAuthRequired_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->credentials)::BaseType::BufferWriter
      credentials_writer;
  mojo::internal::Serialize<::network::mojom::AuthCredentialsDataView>(
      param_credentials, buffer, &credentials_writer, &serialization_context);
  params->credentials.Set(
      credentials_writer.is_null() ? nullptr : credentials_writer.data());
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkServiceClient::OnAuthRequiredCallback* callback) {
    std::move(*callback).Run(
        std::move(param_credentials_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkServiceClient_OnAuthRequired_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->credentials)::BaseType::BufferWriter
      credentials_writer;
  mojo::internal::Serialize<::network::mojom::AuthCredentialsDataView>(
      param_credentials_, buffer, &credentials_writer, serialization_context);
  params->credentials.Set(
      credentials_writer.is_null() ? nullptr : credentials_writer.data());
  }
  base::Optional<net::AuthCredentials> param_credentials_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClient_OnAuthRequired_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceClient_OnAuthRequired_Response_Message::kMessageTag = {};

bool NetworkServiceClient_OnAuthRequired_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkServiceClient::OnAuthRequiredCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkServiceClient_OnAuthRequired_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkServiceClient_OnAuthRequired_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkServiceClient_OnAuthRequired_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  base::Optional<net::AuthCredentials> p_credentials{};
  NetworkServiceClient_OnAuthRequired_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadCredentials(&p_credentials))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkServiceClient::OnAuthRequired response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_credentials));
  return true;
}

void NetworkServiceClient_OnAuthRequired_ProxyToResponder::Run(
    const base::Optional<net::AuthCredentials>& in_credentials) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkServiceClient_OnAuthRequired_Response_Message::Build(kSerialize, is_sync_, std::move(in_credentials));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnAuthRequiredCallback",
               "message", message.name());
#endif
  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkServiceClient_OnCertificateRequested_ProxyToResponder {
 public:
  static NetworkServiceClient::OnCertificateRequestedCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkServiceClient_OnCertificateRequested_ProxyToResponder> proxy(
        new NetworkServiceClient_OnCertificateRequested_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkServiceClient_OnCertificateRequested_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkServiceClient_OnCertificateRequested_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkServiceClient_OnCertificateRequested_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkServiceClient::OnCertificateRequestedCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      const scoped_refptr<net::X509Certificate>& in_x509_certificate, const std::vector<uint16_t>& in_algorithm_preferences, SSLPrivateKeyPtr in_ssl_private_key, bool in_cancel_certificate_selection);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClient_OnCertificateRequested_ProxyToResponder);
};
class NetworkServiceClient_OnCertificateRequested_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceClient_OnCertificateRequested_Response_Message(
      uint32_t message_flags
      , const scoped_refptr<net::X509Certificate>& param_x509_certificate
      , const std::vector<uint16_t>& param_algorithm_preferences
      , SSLPrivateKeyPtr param_ssl_private_key
      , bool param_cancel_certificate_selection
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkServiceClient_OnCertificateRequested_Name,
          message_flags)
      , param_x509_certificate_(std::move(param_x509_certificate))
      , param_algorithm_preferences_(std::move(param_algorithm_preferences))
      , param_ssl_private_key_(param_ssl_private_key.PassInterface())
      , param_cancel_certificate_selection_(std::move(param_cancel_certificate_selection)){}
  ~NetworkServiceClient_OnCertificateRequested_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      const scoped_refptr<net::X509Certificate>& param_x509_certificate, const std::vector<uint16_t>& param_algorithm_preferences, SSLPrivateKeyPtr param_ssl_private_key, bool param_cancel_certificate_selection) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceClient_OnCertificateRequested_Response_Message>(
          kFlags
          , std::move(param_x509_certificate)
          , std::move(param_algorithm_preferences)
          , std::move(param_ssl_private_key)
          , std::move(param_cancel_certificate_selection)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkServiceClient_OnCertificateRequested_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkServiceClient_OnCertificateRequested_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->x509_certificate)::BaseType::BufferWriter
      x509_certificate_writer;
  mojo::internal::Serialize<::network::mojom::X509CertificateDataView>(
      param_x509_certificate, buffer, &x509_certificate_writer, &serialization_context);
  params->x509_certificate.Set(
      x509_certificate_writer.is_null() ? nullptr : x509_certificate_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->x509_certificate.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null x509_certificate in NetworkServiceClient.OnCertificateRequested response");
  typename decltype(params->algorithm_preferences)::BaseType::BufferWriter
      algorithm_preferences_writer;
  const mojo::internal::ContainerValidateParams algorithm_preferences_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<uint16_t>>(
      param_algorithm_preferences, buffer, &algorithm_preferences_writer, &algorithm_preferences_validate_params,
      &serialization_context);
  params->algorithm_preferences.Set(
      algorithm_preferences_writer.is_null() ? nullptr : algorithm_preferences_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->algorithm_preferences.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null algorithm_preferences in NetworkServiceClient.OnCertificateRequested response");
  mojo::internal::Serialize<::network::mojom::SSLPrivateKeyPtrDataView>(
      param_ssl_private_key, &params->ssl_private_key, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->ssl_private_key),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid ssl_private_key in NetworkServiceClient.OnCertificateRequested response");
  params->cancel_certificate_selection = param_cancel_certificate_selection;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkServiceClient::OnCertificateRequestedCallback* callback) {
    std::move(*callback).Run(
        std::move(param_x509_certificate_), 
        std::move(param_algorithm_preferences_), 
        SSLPrivateKeyPtr(std::move(param_ssl_private_key_)), 
        std::move(param_cancel_certificate_selection_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkServiceClient_OnCertificateRequested_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  typename decltype(params->x509_certificate)::BaseType::BufferWriter
      x509_certificate_writer;
  mojo::internal::Serialize<::network::mojom::X509CertificateDataView>(
      param_x509_certificate_, buffer, &x509_certificate_writer, serialization_context);
  params->x509_certificate.Set(
      x509_certificate_writer.is_null() ? nullptr : x509_certificate_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->x509_certificate.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null x509_certificate in NetworkServiceClient.OnCertificateRequested response");
  typename decltype(params->algorithm_preferences)::BaseType::BufferWriter
      algorithm_preferences_writer;
  const mojo::internal::ContainerValidateParams algorithm_preferences_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<uint16_t>>(
      param_algorithm_preferences_, buffer, &algorithm_preferences_writer, &algorithm_preferences_validate_params,
      serialization_context);
  params->algorithm_preferences.Set(
      algorithm_preferences_writer.is_null() ? nullptr : algorithm_preferences_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->algorithm_preferences.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null algorithm_preferences in NetworkServiceClient.OnCertificateRequested response");
  mojo::internal::Serialize<::network::mojom::SSLPrivateKeyPtrDataView>(
      param_ssl_private_key_, &params->ssl_private_key, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->ssl_private_key),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid ssl_private_key in NetworkServiceClient.OnCertificateRequested response");
  params->cancel_certificate_selection = param_cancel_certificate_selection_;
  }
  scoped_refptr<net::X509Certificate> param_x509_certificate_;
  std::vector<uint16_t> param_algorithm_preferences_;
  SSLPrivateKeyPtrInfo param_ssl_private_key_;
  bool param_cancel_certificate_selection_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClient_OnCertificateRequested_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceClient_OnCertificateRequested_Response_Message::kMessageTag = {};

bool NetworkServiceClient_OnCertificateRequested_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkServiceClient::OnCertificateRequestedCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkServiceClient_OnCertificateRequested_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkServiceClient_OnCertificateRequested_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkServiceClient_OnCertificateRequested_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  scoped_refptr<net::X509Certificate> p_x509_certificate{};
  std::vector<uint16_t> p_algorithm_preferences{};
  SSLPrivateKeyPtr p_ssl_private_key{};
  bool p_cancel_certificate_selection{};
  NetworkServiceClient_OnCertificateRequested_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadX509Certificate(&p_x509_certificate))
    success = false;
  if (!input_data_view.ReadAlgorithmPreferences(&p_algorithm_preferences))
    success = false;
  p_ssl_private_key =
      input_data_view.TakeSslPrivateKey<decltype(p_ssl_private_key)>();
  p_cancel_certificate_selection = input_data_view.cancel_certificate_selection();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkServiceClient::OnCertificateRequested response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_x509_certificate), 
std::move(p_algorithm_preferences), 
std::move(p_ssl_private_key), 
std::move(p_cancel_certificate_selection));
  return true;
}

void NetworkServiceClient_OnCertificateRequested_ProxyToResponder::Run(
    const scoped_refptr<net::X509Certificate>& in_x509_certificate, const std::vector<uint16_t>& in_algorithm_preferences, SSLPrivateKeyPtr in_ssl_private_key, bool in_cancel_certificate_selection) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkServiceClient_OnCertificateRequested_Response_Message::Build(kSerialize, is_sync_, std::move(in_x509_certificate), std::move(in_algorithm_preferences), std::move(in_ssl_private_key), std::move(in_cancel_certificate_selection));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnCertificateRequestedCallback",
               "message", message.name());
#endif
  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class NetworkServiceClient_OnSSLCertificateError_ProxyToResponder {
 public:
  static NetworkServiceClient::OnSSLCertificateErrorCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<NetworkServiceClient_OnSSLCertificateError_ProxyToResponder> proxy(
        new NetworkServiceClient_OnSSLCertificateError_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::BindOnce(&NetworkServiceClient_OnSSLCertificateError_ProxyToResponder::Run,
                          std::move(proxy));
  }

  ~NetworkServiceClient_OnSSLCertificateError_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // If we're being destroyed without being run, we want to ensure the
      // binding endpoint has been closed. This checks for that asynchronously.
      // We pass a bound generated callback to handle the response so that any
      // resulting DCHECK stack will have useful interface type information.
      responder_->IsConnectedAsync(base::BindOnce(&OnIsConnectedComplete));
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  NetworkServiceClient_OnSSLCertificateError_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

#if DCHECK_IS_ON()
  static void OnIsConnectedComplete(bool connected) {
    DCHECK(!connected)
        << "NetworkServiceClient::OnSSLCertificateErrorCallback was destroyed without "
        << "first either being run or its corresponding binding being closed. "
        << "It is an error to drop response callbacks which still correspond "
        << "to an open interface pipe.";
  }
#endif

  void Run(
      int32_t in_net_error);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClient_OnSSLCertificateError_ProxyToResponder);
};
class NetworkServiceClient_OnSSLCertificateError_Response_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceClient_OnSSLCertificateError_Response_Message(
      uint32_t message_flags
      , int32_t param_net_error
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkServiceClient_OnSSLCertificateError_Name,
          message_flags)
      , param_net_error_(std::move(param_net_error)){}
  ~NetworkServiceClient_OnSSLCertificateError_Response_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool is_sync,
      int32_t param_net_error) {

    
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceClient_OnSSLCertificateError_Response_Message>(
          kFlags
          , std::move(param_net_error)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkServiceClient_OnSSLCertificateError_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkServiceClient_OnSSLCertificateError_ResponseParams_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->net_error = param_net_error;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkServiceClient::OnSSLCertificateErrorCallback* callback) {
    std::move(*callback).Run(
        std::move(param_net_error_));
  }



 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkServiceClient_OnSSLCertificateError_ResponseParams_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->net_error = param_net_error_;
  }
  int32_t param_net_error_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceClient_OnSSLCertificateError_Response_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceClient_OnSSLCertificateError_Response_Message::kMessageTag = {};

bool NetworkServiceClient_OnSSLCertificateError_ForwardToCallback::Accept(
    mojo::Message* message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "network::mojom::NetworkServiceClient::OnSSLCertificateErrorCallback",
               "message", message->name());
#endif
  mojo::internal::MessageDispatchContext dispatch_context(message);
  if (!message->is_serialized()) {
    auto context =
        message->TakeUnserializedContext<NetworkServiceClient_OnSSLCertificateError_Response_Message>();
    if (!context) {
      // The Message was not of the expected type. It may be a valid message
      // which was build using a different variant of these bindings. Force
      // serialization before dispatch in this case.
      message->SerializeIfNecessary();
    } else {
      if (!callback_.is_null())
        context->Dispatch(&callback_);
      return true;
    }
  }

  DCHECK(message->is_serialized());
  internal::NetworkServiceClient_OnSSLCertificateError_ResponseParams_Data* params =
      reinterpret_cast<
          internal::NetworkServiceClient_OnSSLCertificateError_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  int32_t p_net_error{};
  NetworkServiceClient_OnSSLCertificateError_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_net_error = input_data_view.net_error();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "NetworkServiceClient::OnSSLCertificateError response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_net_error));
  return true;
}

void NetworkServiceClient_OnSSLCertificateError_ProxyToResponder::Run(
    int32_t in_net_error) {
  const bool kSerialize = responder_->PrefersSerializedMessages();
  auto message = NetworkServiceClient_OnSSLCertificateError_Response_Message::Build(kSerialize, is_sync_, std::move(in_net_error));

#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnSSLCertificateErrorCallback",
               "message", message.name());
#endif
  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool NetworkServiceClientStubDispatch::Accept(
    NetworkServiceClient* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kNetworkServiceClient_OnAuthRequired_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnAuthRequired",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkServiceClient_OnCertificateRequested_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnCertificateRequested",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkServiceClient_OnSSLCertificateError_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnSSLCertificateError",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

// static
bool NetworkServiceClientStubDispatch::AcceptWithResponder(
    NetworkServiceClient* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kNetworkServiceClient_OnAuthRequired_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnAuthRequired",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkServiceClientProxy_OnAuthRequired_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkServiceClient::OnAuthRequiredCallback callback =
              NetworkServiceClient_OnAuthRequired_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkServiceClient_OnAuthRequired_Params_Data* params =
          reinterpret_cast<
              internal::NetworkServiceClient_OnAuthRequired_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_process_id{};
      uint32_t p_routing_id{};
      uint32_t p_request_id{};
      int32_t p_resource_type{};
      GURL p_url{};
      bool p_first_auth_attempt{};
      scoped_refptr<net::AuthChallengeInfo> p_auth_info{};
      NetworkServiceClient_OnAuthRequired_ParamsDataView input_data_view(params, &serialization_context);
      
      p_process_id = input_data_view.process_id();
      p_routing_id = input_data_view.routing_id();
      p_request_id = input_data_view.request_id();
      p_resource_type = input_data_view.resource_type();
      if (!input_data_view.ReadUrl(&p_url))
        success = false;
      p_first_auth_attempt = input_data_view.first_auth_attempt();
      if (!input_data_view.ReadAuthInfo(&p_auth_info))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkServiceClient::OnAuthRequired deserializer");
        return false;
      }
      NetworkServiceClient::OnAuthRequiredCallback callback =
          NetworkServiceClient_OnAuthRequired_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnAuthRequired(
std::move(p_process_id), 
std::move(p_routing_id), 
std::move(p_request_id), 
std::move(p_resource_type), 
std::move(p_url), 
std::move(p_first_auth_attempt), 
std::move(p_auth_info), std::move(callback));
      return true;
    }
    case internal::kNetworkServiceClient_OnCertificateRequested_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnCertificateRequested",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkServiceClientProxy_OnCertificateRequested_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkServiceClient::OnCertificateRequestedCallback callback =
              NetworkServiceClient_OnCertificateRequested_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkServiceClient_OnCertificateRequested_Params_Data* params =
          reinterpret_cast<
              internal::NetworkServiceClient_OnCertificateRequested_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_process_id{};
      uint32_t p_routing_id{};
      uint32_t p_request_id{};
      scoped_refptr<net::SSLCertRequestInfo> p_cert_info{};
      NetworkServiceClient_OnCertificateRequested_ParamsDataView input_data_view(params, &serialization_context);
      
      p_process_id = input_data_view.process_id();
      p_routing_id = input_data_view.routing_id();
      p_request_id = input_data_view.request_id();
      if (!input_data_view.ReadCertInfo(&p_cert_info))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkServiceClient::OnCertificateRequested deserializer");
        return false;
      }
      NetworkServiceClient::OnCertificateRequestedCallback callback =
          NetworkServiceClient_OnCertificateRequested_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnCertificateRequested(
std::move(p_process_id), 
std::move(p_routing_id), 
std::move(p_request_id), 
std::move(p_cert_info), std::move(callback));
      return true;
    }
    case internal::kNetworkServiceClient_OnSSLCertificateError_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkServiceClient::OnSSLCertificateError",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkServiceClientProxy_OnSSLCertificateError_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was built using a different variant of these bindings. Force
          // serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          NetworkServiceClient::OnSSLCertificateErrorCallback callback =
              NetworkServiceClient_OnSSLCertificateError_ProxyToResponder::CreateCallback(
                  message->request_id(),
                  message->has_flag(mojo::Message::kFlagIsSync),
                  std::move(responder));
          context->Dispatch(impl, std::move(callback));
          return true;
        }
      }

      internal::NetworkServiceClient_OnSSLCertificateError_Params_Data* params =
          reinterpret_cast<
              internal::NetworkServiceClient_OnSSLCertificateError_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_process_id{};
      uint32_t p_routing_id{};
      uint32_t p_request_id{};
      int32_t p_resource_type{};
      GURL p_url{};
      net::SSLInfo p_ssl_info{};
      bool p_fatal{};
      NetworkServiceClient_OnSSLCertificateError_ParamsDataView input_data_view(params, &serialization_context);
      
      p_process_id = input_data_view.process_id();
      p_routing_id = input_data_view.routing_id();
      p_request_id = input_data_view.request_id();
      p_resource_type = input_data_view.resource_type();
      if (!input_data_view.ReadUrl(&p_url))
        success = false;
      if (!input_data_view.ReadSslInfo(&p_ssl_info))
        success = false;
      p_fatal = input_data_view.fatal();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkServiceClient::OnSSLCertificateError deserializer");
        return false;
      }
      NetworkServiceClient::OnSSLCertificateErrorCallback callback =
          NetworkServiceClient_OnSSLCertificateError_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnSSLCertificateError(
std::move(p_process_id), 
std::move(p_routing_id), 
std::move(p_request_id), 
std::move(p_resource_type), 
std::move(p_url), 
std::move(p_ssl_info), 
std::move(p_fatal), std::move(callback));
      return true;
    }
  }
  return false;
}

bool NetworkServiceClientRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "NetworkServiceClient RequestValidator");

  switch (message->header()->name) {
    case internal::kNetworkServiceClient_OnAuthRequired_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkServiceClient_OnAuthRequired_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkServiceClient_OnCertificateRequested_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkServiceClient_OnCertificateRequested_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkServiceClient_OnSSLCertificateError_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkServiceClient_OnSSLCertificateError_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool NetworkServiceClientResponseValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "NetworkServiceClient ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kNetworkServiceClient_OnAuthRequired_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkServiceClient_OnAuthRequired_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkServiceClient_OnCertificateRequested_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkServiceClient_OnCertificateRequested_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkServiceClient_OnSSLCertificateError_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkServiceClient_OnSSLCertificateError_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
void NetworkServiceClientInterceptorForTesting::OnAuthRequired(uint32_t process_id, uint32_t routing_id, uint32_t request_id, int32_t resource_type, const GURL& url, bool first_auth_attempt, const scoped_refptr<net::AuthChallengeInfo>& auth_info, OnAuthRequiredCallback callback) {
  GetForwardingInterface()->OnAuthRequired(std::move(process_id), std::move(routing_id), std::move(request_id), std::move(resource_type), std::move(url), std::move(first_auth_attempt), std::move(auth_info), std::move(callback));
}
void NetworkServiceClientInterceptorForTesting::OnCertificateRequested(uint32_t process_id, uint32_t routing_id, uint32_t request_id, const scoped_refptr<net::SSLCertRequestInfo>& cert_info, OnCertificateRequestedCallback callback) {
  GetForwardingInterface()->OnCertificateRequested(std::move(process_id), std::move(routing_id), std::move(request_id), std::move(cert_info), std::move(callback));
}
void NetworkServiceClientInterceptorForTesting::OnSSLCertificateError(uint32_t process_id, uint32_t routing_id, uint32_t request_id, int32_t resource_type, const GURL& url, const net::SSLInfo& ssl_info, bool fatal, OnSSLCertificateErrorCallback callback) {
  GetForwardingInterface()->OnSSLCertificateError(std::move(process_id), std::move(routing_id), std::move(request_id), std::move(resource_type), std::move(url), std::move(ssl_info), std::move(fatal), std::move(callback));
}
NetworkServiceClientAsyncWaiter::NetworkServiceClientAsyncWaiter(
    NetworkServiceClient* proxy) : proxy_(proxy) {}

NetworkServiceClientAsyncWaiter::~NetworkServiceClientAsyncWaiter() = default;

void NetworkServiceClientAsyncWaiter::OnAuthRequired(
    uint32_t process_id, uint32_t routing_id, uint32_t request_id, int32_t resource_type, const GURL& url, bool first_auth_attempt, const scoped_refptr<net::AuthChallengeInfo>& auth_info, base::Optional<net::AuthCredentials>* out_credentials) {
  base::RunLoop loop;
  proxy_->OnAuthRequired(std::move(process_id),std::move(routing_id),std::move(request_id),std::move(resource_type),std::move(url),std::move(first_auth_attempt),std::move(auth_info),
      base::BindOnce(
          [](base::RunLoop* loop,
             base::Optional<net::AuthCredentials>* out_credentials
,
             const base::Optional<net::AuthCredentials>& credentials) {*out_credentials = std::move(credentials);
            loop->Quit();
          },
          &loop,
          out_credentials));
  loop.Run();
}
void NetworkServiceClientAsyncWaiter::OnCertificateRequested(
    uint32_t process_id, uint32_t routing_id, uint32_t request_id, const scoped_refptr<net::SSLCertRequestInfo>& cert_info, scoped_refptr<net::X509Certificate>* out_x509_certificate, std::vector<uint16_t>* out_algorithm_preferences, SSLPrivateKeyPtr* out_ssl_private_key, bool* out_cancel_certificate_selection) {
  base::RunLoop loop;
  proxy_->OnCertificateRequested(std::move(process_id),std::move(routing_id),std::move(request_id),std::move(cert_info),
      base::BindOnce(
          [](base::RunLoop* loop,
             scoped_refptr<net::X509Certificate>* out_x509_certificate
,
             std::vector<uint16_t>* out_algorithm_preferences
,
             SSLPrivateKeyPtr* out_ssl_private_key
,
             bool* out_cancel_certificate_selection
,
             const scoped_refptr<net::X509Certificate>& x509_certificate,
             const std::vector<uint16_t>& algorithm_preferences,
             SSLPrivateKeyPtr ssl_private_key,
             bool cancel_certificate_selection) {*out_x509_certificate = std::move(x509_certificate);*out_algorithm_preferences = std::move(algorithm_preferences);*out_ssl_private_key = std::move(ssl_private_key);*out_cancel_certificate_selection = std::move(cancel_certificate_selection);
            loop->Quit();
          },
          &loop,
          out_x509_certificate,
          out_algorithm_preferences,
          out_ssl_private_key,
          out_cancel_certificate_selection));
  loop.Run();
}
void NetworkServiceClientAsyncWaiter::OnSSLCertificateError(
    uint32_t process_id, uint32_t routing_id, uint32_t request_id, int32_t resource_type, const GURL& url, const net::SSLInfo& ssl_info, bool fatal, int32_t* out_net_error) {
  base::RunLoop loop;
  proxy_->OnSSLCertificateError(std::move(process_id),std::move(routing_id),std::move(request_id),std::move(resource_type),std::move(url),std::move(ssl_info),std::move(fatal),
      base::BindOnce(
          [](base::RunLoop* loop,
             int32_t* out_net_error
,
             int32_t net_error) {*out_net_error = std::move(net_error);
            loop->Quit();
          },
          &loop,
          out_net_error));
  loop.Run();
}

const char NetworkService::Name_[] = "network::mojom::NetworkService";

NetworkServiceProxy::NetworkServiceProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}
class NetworkServiceProxy_SetClient_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceProxy_SetClient_Message(
      uint32_t message_flags
      , NetworkServiceClientPtr param_client
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkService_SetClient_Name,
          message_flags)
      , param_client_(param_client.PassInterface()){}
  ~NetworkServiceProxy_SetClient_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      NetworkServiceClientPtr param_client) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceProxy_SetClient_Message>(
          kFlags
          , std::move(param_client)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkService_SetClient_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkService_SetClient_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::NetworkServiceClientPtrDataView>(
      param_client, &params->client, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid client in NetworkService.SetClient request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkService* impl) {
    impl->SetClient(
        NetworkServiceClientPtr(std::move(param_client_)));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkService_SetClient_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::NetworkServiceClientPtrDataView>(
      param_client_, &params->client, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid client in NetworkService.SetClient request");
  }
  NetworkServiceClientPtrInfo param_client_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceProxy_SetClient_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceProxy_SetClient_Message::kMessageTag = {};

void NetworkServiceProxy::SetClient(
    NetworkServiceClientPtr in_client) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkService::SetClient");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkServiceProxy_SetClient_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_client));
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkServiceProxy_CreateNetworkContext_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceProxy_CreateNetworkContext_Message(
      uint32_t message_flags
      , NetworkContextRequest param_context
      , NetworkContextParamsPtr param_params
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkService_CreateNetworkContext_Name,
          message_flags)
      , param_context_(std::move(param_context))
      , param_params_(std::move(param_params)){}
  ~NetworkServiceProxy_CreateNetworkContext_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      NetworkContextRequest param_context, NetworkContextParamsPtr param_params) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceProxy_CreateNetworkContext_Message>(
          kFlags
          , std::move(param_context)
          , std::move(param_params)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkService_CreateNetworkContext_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkService_CreateNetworkContext_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::NetworkContextRequestDataView>(
      param_context, &params->context, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->context),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid context in NetworkService.CreateNetworkContext request");
  typename decltype(params->params)::BaseType::BufferWriter
      params_writer;
  mojo::internal::Serialize<::network::mojom::NetworkContextParamsDataView>(
      param_params, buffer, &params_writer, &serialization_context);
  params->params.Set(
      params_writer.is_null() ? nullptr : params_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->params.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null params in NetworkService.CreateNetworkContext request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkService* impl) {
    impl->CreateNetworkContext(
        std::move(param_context_),
        std::move(param_params_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkService_CreateNetworkContext_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::NetworkContextRequestDataView>(
      param_context_, &params->context, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->context),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid context in NetworkService.CreateNetworkContext request");
  typename decltype(params->params)::BaseType::BufferWriter
      params_writer;
  mojo::internal::Serialize<::network::mojom::NetworkContextParamsDataView>(
      param_params_, buffer, &params_writer, serialization_context);
  params->params.Set(
      params_writer.is_null() ? nullptr : params_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->params.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null params in NetworkService.CreateNetworkContext request");
  }
  NetworkContextRequest param_context_;
  NetworkContextParamsPtr param_params_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceProxy_CreateNetworkContext_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceProxy_CreateNetworkContext_Message::kMessageTag = {};

void NetworkServiceProxy::CreateNetworkContext(
    NetworkContextRequest in_context, NetworkContextParamsPtr in_params) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkService::CreateNetworkContext");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkServiceProxy_CreateNetworkContext_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_context), std::move(in_params));
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkServiceProxy_DisableQuic_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceProxy_DisableQuic_Message(
      uint32_t message_flags
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkService_DisableQuic_Name,
          message_flags){}
  ~NetworkServiceProxy_DisableQuic_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceProxy_DisableQuic_Message>(
          kFlags
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkService_DisableQuic_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkService_DisableQuic_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkService* impl) {
    impl->DisableQuic();
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkService_DisableQuic_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  }

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceProxy_DisableQuic_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceProxy_DisableQuic_Message::kMessageTag = {};

void NetworkServiceProxy::DisableQuic(
    ) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkService::DisableQuic");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkServiceProxy_DisableQuic_Message::Build(
      kSerialize, kExpectsResponse, kIsSync);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkServiceProxy_SetRawHeadersAccess_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceProxy_SetRawHeadersAccess_Message(
      uint32_t message_flags
      , uint32_t param_process_id
      , bool param_allow
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkService_SetRawHeadersAccess_Name,
          message_flags)
      , param_process_id_(std::move(param_process_id))
      , param_allow_(std::move(param_allow)){}
  ~NetworkServiceProxy_SetRawHeadersAccess_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      uint32_t param_process_id, bool param_allow) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceProxy_SetRawHeadersAccess_Message>(
          kFlags
          , std::move(param_process_id)
          , std::move(param_allow)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkService_SetRawHeadersAccess_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkService_SetRawHeadersAccess_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  params->process_id = param_process_id;
  params->allow = param_allow;
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkService* impl) {
    impl->SetRawHeadersAccess(
        std::move(param_process_id_),
        std::move(param_allow_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkService_SetRawHeadersAccess_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  params->process_id = param_process_id_;
  params->allow = param_allow_;
  }
  uint32_t param_process_id_;
  bool param_allow_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceProxy_SetRawHeadersAccess_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceProxy_SetRawHeadersAccess_Message::kMessageTag = {};

void NetworkServiceProxy::SetRawHeadersAccess(
    uint32_t in_process_id, bool in_allow) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkService::SetRawHeadersAccess");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkServiceProxy_SetRawHeadersAccess_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_process_id), std::move(in_allow));
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class NetworkServiceProxy_GetNetworkChangeManager_Message
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  explicit NetworkServiceProxy_GetNetworkChangeManager_Message(
      uint32_t message_flags
      , ::network::mojom::NetworkChangeManagerRequest param_network_change_manager
  )
      : mojo::internal::UnserializedMessageContext(
          &kMessageTag,
          internal::kNetworkService_GetNetworkChangeManager_Name,
          message_flags)
      , param_network_change_manager_(std::move(param_network_change_manager)){}
  ~NetworkServiceProxy_GetNetworkChangeManager_Message() override = default;

  static mojo::Message Build(
      bool serialize,
      bool expects_response,
      bool is_sync,
      ::network::mojom::NetworkChangeManagerRequest param_network_change_manager) {

    
  const uint32_t kFlags =
      ((expects_response) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((is_sync) ? mojo::Message::kFlagIsSync : 0);

    if (!serialize) {
      return mojo::Message(std::make_unique<NetworkServiceProxy_GetNetworkChangeManager_Message>(
          kFlags
          , std::move(param_network_change_manager)
          ));
    }

    DCHECK(serialize);
    
  mojo::Message message(
      internal::kNetworkService_GetNetworkChangeManager_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::network::mojom::internal::NetworkService_GetNetworkChangeManager_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::NetworkChangeManagerRequestDataView>(
      param_network_change_manager, &params->network_change_manager, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->network_change_manager),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid network_change_manager in NetworkService.GetNetworkChangeManager request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);
    return message;
  }


  void Dispatch(NetworkService* impl) {
    impl->GetNetworkChangeManager(
        std::move(param_network_change_manager_));
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* serialization_context,
                 mojo::internal::Buffer* buffer) override {
    ::network::mojom::internal::NetworkService_GetNetworkChangeManager_Params_Data::BufferWriter
        params;
    params.Allocate(buffer);
  mojo::internal::Serialize<::network::mojom::NetworkChangeManagerRequestDataView>(
      param_network_change_manager_, &params->network_change_manager, serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->network_change_manager),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid network_change_manager in NetworkService.GetNetworkChangeManager request");
  }
  ::network::mojom::NetworkChangeManagerRequest param_network_change_manager_;

  DISALLOW_COPY_AND_ASSIGN(NetworkServiceProxy_GetNetworkChangeManager_Message);
};

const mojo::internal::UnserializedMessageContext::Tag
NetworkServiceProxy_GetNetworkChangeManager_Message::kMessageTag = {};

void NetworkServiceProxy::GetNetworkChangeManager(
    ::network::mojom::NetworkChangeManagerRequest in_network_change_manager) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "network::mojom::NetworkService::GetNetworkChangeManager");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  const bool kSerialize = receiver_->PrefersSerializedMessages();
  auto message = NetworkServiceProxy_GetNetworkChangeManager_Message::Build(
      kSerialize, kExpectsResponse, kIsSync, std::move(in_network_change_manager));
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool NetworkServiceStubDispatch::Accept(
    NetworkService* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kNetworkService_SetClient_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::SetClient",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkServiceProxy_SetClient_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkService_SetClient_Params_Data* params =
          reinterpret_cast<internal::NetworkService_SetClient_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      NetworkServiceClientPtr p_client{};
      NetworkService_SetClient_ParamsDataView input_data_view(params, &serialization_context);
      
      p_client =
          input_data_view.TakeClient<decltype(p_client)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkService::SetClient deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetClient(
std::move(p_client));
      return true;
    }
    case internal::kNetworkService_CreateNetworkContext_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::CreateNetworkContext",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkServiceProxy_CreateNetworkContext_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkService_CreateNetworkContext_Params_Data* params =
          reinterpret_cast<internal::NetworkService_CreateNetworkContext_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      NetworkContextRequest p_context{};
      NetworkContextParamsPtr p_params{};
      NetworkService_CreateNetworkContext_ParamsDataView input_data_view(params, &serialization_context);
      
      p_context =
          input_data_view.TakeContext<decltype(p_context)>();
      if (!input_data_view.ReadParams(&p_params))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkService::CreateNetworkContext deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateNetworkContext(
std::move(p_context), 
std::move(p_params));
      return true;
    }
    case internal::kNetworkService_DisableQuic_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::DisableQuic",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkServiceProxy_DisableQuic_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkService_DisableQuic_Params_Data* params =
          reinterpret_cast<internal::NetworkService_DisableQuic_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      NetworkService_DisableQuic_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkService::DisableQuic deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->DisableQuic();
      return true;
    }
    case internal::kNetworkService_SetRawHeadersAccess_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::SetRawHeadersAccess",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkServiceProxy_SetRawHeadersAccess_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkService_SetRawHeadersAccess_Params_Data* params =
          reinterpret_cast<internal::NetworkService_SetRawHeadersAccess_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_process_id{};
      bool p_allow{};
      NetworkService_SetRawHeadersAccess_ParamsDataView input_data_view(params, &serialization_context);
      
      p_process_id = input_data_view.process_id();
      p_allow = input_data_view.allow();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkService::SetRawHeadersAccess deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetRawHeadersAccess(
std::move(p_process_id), 
std::move(p_allow));
      return true;
    }
    case internal::kNetworkService_GetNetworkChangeManager_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::GetNetworkChangeManager",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);
      if (!message->is_serialized()) {
        auto context = message->TakeUnserializedContext<
            NetworkServiceProxy_GetNetworkChangeManager_Message>();
        if (!context) {
          // The Message was not of the expected type. It may be a valid message
          // which was serialized using a different variant of these bindings.
          // Force serialization before dispatch in this case.
          message->SerializeIfNecessary();
        } else {
          context->Dispatch(impl);
          return true;
        }
      }

      DCHECK(message->is_serialized());
      internal::NetworkService_GetNetworkChangeManager_Params_Data* params =
          reinterpret_cast<internal::NetworkService_GetNetworkChangeManager_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::network::mojom::NetworkChangeManagerRequest p_network_change_manager{};
      NetworkService_GetNetworkChangeManager_ParamsDataView input_data_view(params, &serialization_context);
      
      p_network_change_manager =
          input_data_view.TakeNetworkChangeManager<decltype(p_network_change_manager)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "NetworkService::GetNetworkChangeManager deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetNetworkChangeManager(
std::move(p_network_change_manager));
      return true;
    }
  }
  return false;
}

// static
bool NetworkServiceStubDispatch::AcceptWithResponder(
    NetworkService* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kNetworkService_SetClient_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::SetClient",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkService_CreateNetworkContext_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::CreateNetworkContext",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkService_DisableQuic_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::DisableQuic",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkService_SetRawHeadersAccess_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::SetRawHeadersAccess",
               "message", message->name());
#endif
      break;
    }
    case internal::kNetworkService_GetNetworkChangeManager_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)network::mojom::NetworkService::GetNetworkChangeManager",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool NetworkServiceRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "NetworkService RequestValidator");

  switch (message->header()->name) {
    case internal::kNetworkService_SetClient_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkService_SetClient_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkService_CreateNetworkContext_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkService_CreateNetworkContext_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkService_DisableQuic_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkService_DisableQuic_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkService_SetRawHeadersAccess_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkService_SetRawHeadersAccess_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kNetworkService_GetNetworkChangeManager_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::NetworkService_GetNetworkChangeManager_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

void NetworkServiceInterceptorForTesting::SetClient(NetworkServiceClientPtr client) {
  GetForwardingInterface()->SetClient(std::move(client));
}
void NetworkServiceInterceptorForTesting::CreateNetworkContext(NetworkContextRequest context, NetworkContextParamsPtr params) {
  GetForwardingInterface()->CreateNetworkContext(std::move(context), std::move(params));
}
void NetworkServiceInterceptorForTesting::DisableQuic() {
  GetForwardingInterface()->DisableQuic();
}
void NetworkServiceInterceptorForTesting::SetRawHeadersAccess(uint32_t process_id, bool allow) {
  GetForwardingInterface()->SetRawHeadersAccess(std::move(process_id), std::move(allow));
}
void NetworkServiceInterceptorForTesting::GetNetworkChangeManager(::network::mojom::NetworkChangeManagerRequest network_change_manager) {
  GetForwardingInterface()->GetNetworkChangeManager(std::move(network_change_manager));
}
NetworkServiceAsyncWaiter::NetworkServiceAsyncWaiter(
    NetworkService* proxy) : proxy_(proxy) {}

NetworkServiceAsyncWaiter::~NetworkServiceAsyncWaiter() = default;


}  // namespace mojom
}  // namespace network

namespace mojo {


// static
bool StructTraits<::network::mojom::NetworkContextParams::DataView, ::network::mojom::NetworkContextParamsPtr>::Read(
    ::network::mojom::NetworkContextParams::DataView input,
    ::network::mojom::NetworkContextParamsPtr* output) {
  bool success = true;
  ::network::mojom::NetworkContextParamsPtr result(::network::mojom::NetworkContextParams::New());
  
      if (!input.ReadContextName(&result->context_name))
        success = false;
      if (!input.ReadUserAgent(&result->user_agent))
        success = false;
      if (!input.ReadAcceptLanguage(&result->accept_language))
        success = false;
      result->enable_brotli = input.enable_brotli();
      if (!input.ReadQuicUserAgentId(&result->quic_user_agent_id))
        success = false;
      result->proxy_resolver_factory =
          input.TakeProxyResolverFactory<decltype(result->proxy_resolver_factory)>();
      if (!input.ReadCookiePath(&result->cookie_path))
        success = false;
      result->restore_old_session_cookies = input.restore_old_session_cookies();
      result->persist_session_cookies = input.persist_session_cookies();
      if (!input.ReadChannelIdPath(&result->channel_id_path))
        success = false;
      result->http_cache_enabled = input.http_cache_enabled();
      result->http_cache_max_size = input.http_cache_max_size();
      if (!input.ReadHttpCachePath(&result->http_cache_path))
        success = false;
      if (!input.ReadHttpServerPropertiesPath(&result->http_server_properties_path))
        success = false;
      result->enable_data_url_support = input.enable_data_url_support();
      result->enable_file_url_support = input.enable_file_url_support();
      result->enable_ftp_url_support = input.enable_ftp_url_support();
      result->http_09_on_non_default_ports_enabled = input.http_09_on_non_default_ports_enabled();
      if (!input.ReadInitialProxyConfig(&result->initial_proxy_config))
        success = false;
      result->proxy_config_client_request =
          input.TakeProxyConfigClientRequest<decltype(result->proxy_config_client_request)>();
      result->proxy_config_poller_client =
          input.TakeProxyConfigPollerClient<decltype(result->proxy_config_poller_client)>();
      result->allow_gssapi_library_load = input.allow_gssapi_library_load();
      if (!input.ReadGssapiLibraryName(&result->gssapi_library_name))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::network::mojom::NetworkConditions::DataView, ::network::mojom::NetworkConditionsPtr>::Read(
    ::network::mojom::NetworkConditions::DataView input,
    ::network::mojom::NetworkConditionsPtr* output) {
  bool success = true;
  ::network::mojom::NetworkConditionsPtr result(::network::mojom::NetworkConditions::New());
  
      result->offline = input.offline();
      if (!input.ReadLatency(&result->latency))
        success = false;
      result->download_throughput = input.download_throughput();
      result->upload_throughput = input.upload_throughput();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::network::mojom::ClearCacheUrlFilter::DataView, ::network::mojom::ClearCacheUrlFilterPtr>::Read(
    ::network::mojom::ClearCacheUrlFilter::DataView input,
    ::network::mojom::ClearCacheUrlFilterPtr* output) {
  bool success = true;
  ::network::mojom::ClearCacheUrlFilterPtr result(::network::mojom::ClearCacheUrlFilter::New());
  
      if (!input.ReadType(&result->type))
        success = false;
      if (!input.ReadDomains(&result->domains))
        success = false;
      if (!input.ReadOrigins(&result->origins))
        success = false;
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif